/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.util.*;
import java.sql.*;
import org.apache.log4j.*;
import com.silverwrist.util.*;
import com.silverwrist.venice.*;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.core.internals.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.security.*;

class UserContextImpl implements UserContext, UserBackend
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  // Property set indexes
  private static final int PROP_FLAGS = 0;  // flags

  private static final BitSet ALL_PROPS;    // BitSet representing all properties

  // Boolean flag indexes
  private static final int BF_POSTPICTURES = 0;

  private static Category logger = Category.getInstance(UserContextImpl.class);

  private static final String AUTH_TOKEN_PREFIX = "VQAT:";
  private static final char AUTH_TOKEN_SEP = '|';

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private EnvUser env;                     // the environment store
  private String remote_addr;              // remote address identifier
  private int uid = -1;                    // the user ID we're using
  private int contactid;                   // ID of our contact information
  private int level;                       // base security level for this user
  private int confirm_num;                 // email confirmation number for this user
  private boolean is_anon = false;         // is this the anonymous user?
  private boolean email_verified = false;  // has email address been verified?
  private String username;                 // the user name we're using
  private java.util.Date created;          // when was this user created? (GMT)
  private java.util.Date last_access;      // when did we last log in? (GMT)
  private String description;              // personal description
  private String my_email = null;          // my email address (cached)
  private String my_pseud = null;          // my pseud (cached)
  private String full_name = null;         // my full name (cached)
  private Locale my_locale = null;         // my default locale (cached)
  private TimeZone my_tz = null;           // my default timezone (cached)
  private HashMap mru_cache = new HashMap();  // MRU cache for data objects
  private OptionSet flags = null;             // option flags

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  UserContextImpl(EnvEngine env)
  {
    this.env = new EnvUser(env,this);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private void loadUserData(ResultSet rs) throws SQLException
  {
    uid = rs.getInt("uid");
    username = rs.getString("username");
    // skip field "passhash"
    contactid = rs.getInt("contactid");
    is_anon = rs.getBoolean("is_anon");
    email_verified = rs.getBoolean("verify_email");
    // skip fields "lockout", "access_tries"
    confirm_num = rs.getInt("email_confnum");
    level = rs.getInt("base_lvl");
    created = SQLUtil.getFullDateTime(rs,"created");
    last_access = SQLUtil.getFullDateTime(rs,"lastaccess");
    // skip field "passreminder"
    description = rs.getString("description");

    // purge any "cached" fields that may be left over
    my_email = null;
    my_pseud = null;
    full_name = null;
    my_locale = null;
    my_tz = null;

    if (logger.isDebugEnabled())
    { // spit it all out to debug info
      logger.debug("Loaded: UID " + uid + ", username \"" + username + "\", contactid " + contactid);
      logger.debug("...is_anon " + is_anon + ", email_verified " + email_verified);
      logger.debug("... level " + level);

    } // end if

  } // end loadUserData

  private void loadPrefs() throws DataException
  {
    Connection conn = null;

    try
    { // call through to lower level function
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery("SELECT * FROM userprefs WHERE uid = " + uid + ";");
      
      if (!(rs.next()))
	throw new DataException("cannot find preferences for user");

      if (my_tz==null)
	my_tz = TimeZone.getTimeZone(rs.getString("tzid"));
      
      if (my_locale==null)
	my_locale = International.get().createLocale(rs.getString("localeid"));

      // Load the user properties as well.
      rs = stmt.executeQuery("SELECT ndx, data FROM propuser WHERE uid = " + uid + ";");
      while (rs.next())
      { // load the properties...
	switch (rs.getInt(1))
	{ // based on the property, do what is needed
	  case PROP_FLAGS:
	    flags = new OptionSet(rs.getString(2));
	    break;

	  default:
	    break;

	} // end switch

      } // end while

    } // end try
    catch (SQLException e)
    { // translate into DataException, yadda yadda yadda
      logger.error("SQL error reading preferences: " + e.getMessage(),e);
      throw new DataException("Unable to read user preferences: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection
      env.releaseConnection(conn);

    } // end finally

  } // end loadPrefs

  private void sendEmailConfirmation() throws DataException, EmailException
  {
    if (logger.isDebugEnabled())
      logger.debug("sendEmailConfirmation(): sending to \"" + my_email + "\"");

    // Create the message to be sent.
    String message = env.getEngine().getStockMessage("email-confirm");
    if (message==null)
    { // no message defined?  oy!
      logger.error("internal error condition: email-confirm stock message not defined");
      throw new DataException("INTERNAL: email-confirm message not defined");

    } // end if

    // Replace the variables in the text.
    HashMap vars = new HashMap(2);
    vars.put("username",username);
    vars.put("confnum",String.valueOf(confirm_num));
    message = StringUtil.replaceAllVariables(message,vars);

    String subject = env.getEngine().getStockMessage("email-confirm-subject");
    if (subject==null)
      subject = "Venice Email Confirmation";

    // Create the emailer and send the message.
    Emailer emailer = env.getEngine().createEmailer();
    emailer.setTo(my_email);
    emailer.setSubject(subject);
    emailer.setText(message);
    emailer.send();
    if (logger.isDebugEnabled())
      logger.debug("...email sent!");

  } // end sendEmailConfirmation

  private void autoJoinCommunities(Connection conn) throws SQLException
  {
    if (logger.isDebugEnabled())
      logger.debug("autoJoinCommunities (uid " + uid + ", level " + level + ")");

    // See which communities we are eligible to autojoin.
    Statement stmt = conn.createStatement();
    StringBuffer sql =
        new StringBuffer("SELECT sigmember.sigid, sigmember.locked FROM users, sigmember, sigs "
			 + "WHERE sigmember.uid = users.uid AND sigmember.sigid = sigs.sigid "
			 + "AND users.is_anon = 1 AND sigs.join_lvl <= ");
    sql.append(level).append(';');
    if (logger.isDebugEnabled())
      logger.debug("SQL: " + sql.toString());
    ResultSet rs = stmt.executeQuery(sql.toString());

    // Save the community IDs returned into temporary array lists.
    ArrayList tmp_cid = new ArrayList();
    ArrayList tmp_locked = new ArrayList();
    while (rs.next())
    { // save off the "cid" and "locked" column pairs
      tmp_cid.add(new Integer(rs.getInt(1)));
      tmp_locked.add(new Boolean(rs.getBoolean(2)));

    } // end while

    // Figure out which of those communities we haven't joined yet and set up to autojoin them.
    sql.setLength(0);
    int new_level = env.getCommunityDefaultRole("Community.NewUser").getLevel();
    for (int i=0; i<tmp_cid.size(); i++)
    { // see if the user is already a member of this community
      Integer x_cid = (Integer)(tmp_cid.get(i));
      rs = stmt.executeQuery("SELECT sigid FROM sigmember WHERE sigid = " + x_cid + " AND uid = " + uid
			     + ";");
      if (!(rs.next()))
      { // tack this information onto the end of our big "INSERT" command
	Boolean x_locked = (Boolean)(tmp_locked.get(i));
	if (sql.length()==0)
	  sql.append("INSERT INTO sigmember (sigid, uid, granted_lvl, locked) VALUES ");
	else
	  sql.append(", ");
	sql.append("(").append(x_cid).append(", ").append(uid).append(", ").append(new_level).append(", ");
	sql.append(x_locked.booleanValue() ? '1' : '0').append(")");
				 
      } // end if

    } // end for

    if (sql.length()>0)
    { // execute the big update
      sql.append(';');
      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());
      stmt.executeUpdate(sql.toString());

    } // end if

  } // end autoJoinCommunities

  private void updateProperties(BitSet delta) throws DataException
  {
    Connection conn = null;
    try
    { // get a connection and create a statement
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer();

      if (delta.get(PROP_FLAGS))
      { // store the flags
	sql.append("UPDATE propuser SET data = '").append(flags.asString()).append("' WHERE uid = ");
	sql.append(uid).append(" AND ndx = ").append(PROP_FLAGS).append(';');
	stmt.executeUpdate(sql.toString());
	sql.setLength(0);

      } // end if

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error storing community properties: " + e.getMessage(),e);
      throw new DataException("unable to save community properties: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end updateProperties

  private void updateProperties() throws DataException
  {
    updateProperties(ALL_PROPS);

  } // end updateProperties

  private final UserProperties createProperties()
  {
    UserProperties rc = new UserProperties();
    rc.setDisplayPostPictures(flags.get(BF_POSTPICTURES));
    return rc;

  } // end createProperties

  private final BitSet storeProperties(UserProperties props)
  {
    BitSet rc = new BitSet();

    if (flags.assign(BF_POSTPICTURES,props.getDisplayPostPictures()))
      rc.set(PROP_FLAGS);

    return rc;

  } // end storeProperties

  /*--------------------------------------------------------------------------------
   * Implementations from interface UserContext
   *--------------------------------------------------------------------------------
   */

  public int getUID()
  {
    return uid;

  } // end getUID

  public String getUserName()
  {
    return username;

  } // end getUserName

  public int getContactID()
  {
    return contactid;

  } // end getContactID

  public String getDescription()
  {
    return description;

  } // end getDescription

  public boolean isLoggedIn()
  {
    if (logger.isDebugEnabled())
      logger.debug("isLoggedIn(): uid = " + uid + ", is_anon = " + is_anon);
    return ((uid!=-1) && !is_anon);

  } // end is_logged_in

  public boolean isEmailVerified()
  {
    return email_verified;

  } // end isEmailVerified

  public void authenticate(String username, String password) throws AccessError, DataException
  {
    if (isLoggedIn())
    { // already authenticated, can't authenticate again
      logger.error("UserContext already authenticated (with uid " + uid + ")");
      throw new InternalStateError("context already authenticated");

    } // end if

    if (logger.isDebugEnabled())
      logger.debug("authenticate(): authenticating user \"" + username + "\"...");

    Connection conn = null;
    AuditRecord ar = null;
      
    try
    { // look for a user name matching this user record
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery("SELECT * FROM users WHERE username = '"
				       + SQLUtil.encodeString(username) + "';");

      if (!(rs.next()))
      {  // user not found
	logger.error("...user not found");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,0,remote_addr,"Bad username: " + username);
	throw new AccessError("The user account you have specified does not exist.  Please try again.");

      } // end if

      int the_uid = rs.getInt("uid");

      if (rs.getBoolean("is_anon"))
      { // can't log in as Anonymous Honyak
	logger.error("...user is the Anonymous Honyak, can't explicitly login");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,the_uid,remote_addr,"Anonymous user");
	throw new AccessError("This account cannot be explicitly logged into.  Please try again.");

      } // end if

      if (rs.getBoolean("lockout"))
      { // account locked out
	logger.error("...user is locked out by the Admin");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,the_uid,remote_addr,"Account locked out");
	throw new AccessError("This account has been administratively locked out.  Please contact "
			      + "the system administrator for assistance.");

      } // end if

      // hash the password and compare it to the stored password hash
      PasswordHash phash = new PasswordHash(password);
      if (!(phash.equals(rs.getString("passhash"))))
      { // they specified an incorrect password
	// TODO: advance an "invalid" counter and lock the account out if more than X tries
	logger.warn("...invalid password");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,the_uid,remote_addr,"Bad password");
	throw new AccessError("The password you have specified is incorrect.  Please try again.");

      } // end if

      if (logger.isDebugEnabled())
	logger.debug("...authenticated");

      // we're authenticated - load the user data into the context
      loadUserData(rs);

      // update the "last access" time in the database
      java.util.Date mydate = new java.util.Date();
      stmt.executeUpdate("UPDATE users SET lastaccess = '" + SQLUtil.encodeDate(mydate)
			 + "' WHERE uid = " + uid + ";");

      // update the "last access" time in this object
      last_access = mydate;

      // an audit record indicating we logged in OK
      ar = new AuditRecord(AuditRecord.LOGIN_OK,the_uid,remote_addr);

      if (logger.isDebugEnabled())
	logger.debug("...context loaded, we're ready :-)");

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error reading user data: " + e.getMessage(),e);
      throw new DataException("unable to access user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end if

  } // end authenticate

  public void confirmEmail(int conf_num) throws AccessError, DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("confirmEmail(): confirming for UID " + uid);
    if ((email_verified) || env.testPermission(EnvUser.PERM_NOEMAILVERIFY))
    { // already confirmed
      if (logger.isDebugEnabled())
	logger.debug("...user has either already confirmed or is exempt");
      return;

    } // end if

    AuditRecord ar = null;

    if (conf_num!=confirm_num)
    { // the confirmation number is wrong
      logger.warn("...confirmation number incorrect");
      ar = new AuditRecord(AuditRecord.VERIFY_FAIL,uid,remote_addr,"Invalid confirmation number");
      env.getEngine().saveAuditRecord(ar);
      throw new AccessError("Confirmation number is incorrect.  Please try again.");

    } // end if

    Connection conn = null;
    Role new_role = env.getDefaultRole("Global.AfterVerify");

    try
    { // get a connection and set the user's status to reflect the verification
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE users SET verify_email = 1, base_lvl = ");
      sql.append(new_role.getLevel()).append(" WHERE uid = ").append(uid).append(';');
      stmt.executeUpdate(sql.toString());
      email_verified = true;
      level = new_role.getLevel();

      autoJoinCommunities(conn);  // EJB 4/14/2001 - handle autojoin of any SIGs we couldn't autojoin at
                                  // account creation time

      // record an audit message indicating that we verified OK
      ar = new AuditRecord(AuditRecord.VERIFY_OK,uid,remote_addr);

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error updating user data: " + e.getMessage(),e);
      throw new DataException("unable to update user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end if

  } // end confirmEmail

  public void resendEmailConfirmation() throws DataException, EmailException
  {
    if (logger.isDebugEnabled())
      logger.debug("resendEmailConfirmation(): resending for UID " + uid);
    if ((email_verified) || env.testPermission(EnvUser.PERM_NOEMAILVERIFY))
    { // already confirmed, no need to resend
      if (logger.isDebugEnabled())
	logger.debug("...user has either already confirmed or is exempt");
      return;

    } // end if

    if (my_email==null)
      getContactInfo();  // forces my_email to be updated

    Connection conn = null;
    AuditRecord ar = null;
    try
    { // need to change the user's email confirmation number first
      conn = env.getConnection();
      Statement stmt = conn.createStatement();

      // generate new confirmation number
      int new_confirm_num = env.getEngine().getNewConfirmationNumber();

      // create an SQL statement to reset the user account information, and execute it
      StringBuffer sql = new StringBuffer("UPDATE users SET email_confnum = ");
      sql.append(new_confirm_num).append(" WHERE uid = ").append(uid).append(';');
      stmt.executeUpdate(sql.toString());

      confirm_num = new_confirm_num;  // save changed value

      // now send the email confirmation!
      sendEmailConfirmation();

      // log that a resend was performed...
      ar = new AuditRecord(AuditRecord.RESEND_CONFIRM,uid,remote_addr);

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error updating user data: " + e.getMessage(),e);
      throw new DataException("unable to update user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end if
    
  } // end resendEmailConfirmation

  public ContactInfo getContactInfo() throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("getContactInfo() for UID " + uid);

    ContactInfoImpl rc;
    if (contactid>=0)
      rc = new ContactInfoImpl(env,contactid);
    else
      rc = new ContactInfoImpl(uid);
    if (my_email==null)
      my_email = rc.getEmail();
    if (full_name==null)
      full_name = rc.getGivenName() + " " + rc.getFamilyName();
    if (my_pseud==null)
      my_pseud = full_name;
    return rc;

  } // end getContactInfo

  public boolean putContactInfo(ContactInfo ci) throws DataException, EmailException
  {
    if (logger.isDebugEnabled())
      logger.debug("putContactInfo() for UID " + uid);

    boolean email_changed = false;
    if ((ci.getOwnerUID()!=uid) || (ci.getOwnerCommunityID()>=0))
    { // the contact information is not owned correctly
      logger.error("ContactInfo ownership wrong (it's " + ci.getOwnerUID() + ", " + ci.getOwnerCommunityID()
		   + "), should be (" + uid + ", -1)");
      throw new DataException("invalid contact information record");

    } // end if

    if (is_anon)
    { // we can't change Anonymous Honyak this way!
      logger.error("cannot modify anonymous user contact info");
      throw new DataException("unmodifiable contact information");

    } // end if

    Connection conn = null;  // database connection
    AuditRecord ar = null;   // audit record
      
    try
    { // get a database connection
      conn = env.getConnection();
      Stashable obj = (Stashable)ci;

      // save the contact information
      obj.stash(conn);

      if (contactid<0)
      { // contact being established for the first time
	contactid = ci.getContactID();
	if (logger.isDebugEnabled())
	  logger.debug("...established initial contact (" + contactid + ") for user");
	my_email = ci.getEmail();
	sendEmailConfirmation();
	email_changed = true;

      } // end if

      full_name = ci.getGivenName() + " " + ci.getFamilyName();  // update this field
      my_pseud = full_name;

      if (my_email==null)  // filling in, this is not necessarily the first time
	my_email = ci.getEmail();
      else if (!(my_email.equals(ci.getEmail())) && !(env.testPermission(EnvUser.PERM_NOEMAILVERIFY)))
      { // email address change - need to reconfirm - but choose a new confirmation
	// number and save it in the database first; also turn off the verify flag
	// and set the base level to GLOBAL_UNVERIFIED
	// NOTE: Administrators are exempt from this requirement!

	if (logger.isDebugEnabled())
	  logger.debug("email address changed, need to reconfirm");

	// generate new confirmation number
	int new_confirm_num = env.getEngine().getNewConfirmationNumber();
	Role new_role = env.getDefaultRole("Global.Unverified");

	// create an SQL statement to reset the user account information, and execute it
	StringBuffer sql = new StringBuffer("UPDATE users SET verify_email = 0, email_confnum = ");
	sql.append(new_confirm_num).append(", base_lvl = ").append(new_role.getLevel());
	sql.append(" WHERE uid = ").append(uid).append(';');
	Statement stmt = conn.createStatement();
	stmt.executeUpdate(sql.toString());

	// save off changed data
	email_verified = false;
	confirm_num = new_confirm_num;
	level = new_role.getLevel();

	// now send the email confirmation!
	sendEmailConfirmation();

	email_changed = true;

      } // end else if

      ar = new AuditRecord(AuditRecord.USER_CONTACT_INFO,uid,remote_addr,"contactid=" + contactid);

    } // end try
    catch (ClassCastException cce)
    { // we need to be able to coerce the ContactInfo to a Stashable
      logger.error("ContactInfo needs to be a Stashable for this to work");
      throw new DataException("improper contact information record");

    } // end catch
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error updating contact info: " + e.getMessage(),e);
      throw new DataException("unable to access user contact data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end if

    return email_changed;

  } // end putContactInfo

  public UserProfile getProfile(String xusername) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("getProfile(\"" + xusername + "\")...");
    Connection conn = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      UserProfileImpl prof = new UserProfileImpl(env,conn,xusername);
      if (logger.isDebugEnabled())
	logger.debug("...found it!");
      return prof;

    } // end catch
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error looking up profile information: " + e.getMessage(),e);
      throw new DataException("Unable to look up user info: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end getProfile

  public UserProfile getProfile(int xuid) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("getProfile(#" + xuid + ")...");
    Connection conn = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      UserProfileImpl prof = new UserProfileImpl(env,conn,xuid);
      if (logger.isDebugEnabled())
	logger.debug("...found it!");
      return prof;

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error looking up profile information: " + e.getMessage(),e);
      throw new DataException("Unable to look up user info: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end getProfile

  public void setPassword(String password, String reminder) throws DataException
  {
    if (is_anon)
    { // trying to change Anonymous Honyak's (nonexistent) password?
      logger.error("cannot change password of anonymous account");
      throw new DataException("The anonymous account cannot change its password.");

    } // end if

    Connection conn = null;
    AuditRecord ar = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      PasswordHash phash = new PasswordHash(password);
      StringBuffer sql = new StringBuffer("UPDATE users SET passhash = '");
      sql.append(phash.toString()).append("', passreminder = '").append(SQLUtil.encodeString(reminder));
      sql.append("' WHERE uid = ").append(uid).append(';');
      stmt.executeUpdate(sql.toString());

      // record an audit record for this user
      ar = new AuditRecord(AuditRecord.PASSWORD_CHANGE,uid,remote_addr);
      
    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error changing password: " + e.getMessage(),e);
      throw new DataException("Unable to set user password: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end finally

  } // end setPassword

  public void setDescription(String new_descr) throws DataException
  {
    if (is_anon)
    { // trying to change Anonymous Honyak's (nonexistent) description?
      logger.error("cannot change description of anonymous account");
      throw new DataException("The anonymous account cannot change its description.");

    } // end if

    Connection conn = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE users SET description = '");
      sql.append(SQLUtil.encodeString(new_descr)).append("' WHERE uid = ").append(uid).append(';');
      stmt.executeUpdate(sql.toString());

      description = new_descr;  // change stored information
      
    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error changing description: " + e.getMessage(),e);
      throw new DataException("Unable to set user description: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end setDescription

  public List getMemberCommunities() throws DataException
  {
    return CommunityUserContextImpl.getMemberCommunityEntries(env);

  } // end getMemberCommunities

  public CommunityContext getCommunityContext(int cid) throws DataException
  {
    return CommunityUserContextImpl.getCommunityContext(env,cid);

  } // end getCommunityContext

  public CommunityContext getCommunityContext(String alias) throws DataException
  {
    return CommunityUserContextImpl.getCommunityContext(env,alias);

  } // end getCommunityContext

  public List getRootCategoryList() throws DataException
  {
    return CategoryDescriptorImpl.getTopLevelCategoryList(env,env.testPermission(EnvUser.PERM_SHOWHIDDENCATS));

  } // end getRootCategoryList

  public CategoryDescriptor getCategoryDescriptor(int catid) throws DataException
  {
    return new CategoryDescriptorImpl(env,catid,!(env.testPermission(EnvUser.PERM_SHOWHIDDENCATS)));

  } // end getCategoryDescriptor

  public List searchForCommunities(int field, int mode, String term, int offset, int count)
      throws DataException
  {
    return CommunityUserContextImpl.searchForCommunities(env,field,mode,term,offset,count);

  } // end searchForCommunities

  public int getSearchCommunityCount(int field, int mode, String term) throws DataException
  {
    return CommunityUserContextImpl.getSearchCommunityCount(env,field,mode,term);

  } // end getSearchCommunityCount

  public List getCommunitiesInCategory(int catid, int offset, int count) throws DataException
  {
    return CommunityUserContextImpl.getCommunitiesInCategory(env,catid,offset,count);

  } // end getCommunitiesInCategory

  public List getCommunitiesInCategory(CategoryDescriptor cat, int offset, int count) throws DataException
  {
    return CommunityUserContextImpl.getCommunitiesInCategory(env,cat.getLinkedCategoryID(),offset,count);

  } // end getCommunitiesInCategory

  public int getNumCommunitiesInCategory(int catid) throws DataException
  {
    return CommunityUserContextImpl.getNumCommunitiesInCategory(env,catid);

  } // end getNumCommunitiessInCategory

  public int getNumCommunitiesInCategory(CategoryDescriptor cat) throws DataException
  {
    return CommunityUserContextImpl.getNumCommunitiesInCategory(env,cat.getLinkedCategoryID());

  } // end getNumCommunitiesInCategory

  public List searchForCategories(int mode, String term, int offset, int count) throws DataException
  {
    return CategoryDescriptorImpl.searchForCategories(env,env.testPermission(EnvUser.PERM_SHOWHIDDENCATS),
						      env.testPermission(EnvUser.PERM_SEARCHHIDDENCATS),mode,
						      term,offset,count);

  } // end searchForCategories

  public int getSearchCategoryCount(int mode, String term) throws DataException
  {
    return CategoryDescriptorImpl.getSearchCategoryCount(env,env.testPermission(EnvUser.PERM_SHOWHIDDENCATS),
							 env.testPermission(EnvUser.PERM_SEARCHHIDDENCATS),
							 mode,term);

  } // end getSearchCategoryCount

  public CommunityContext createCommunity(String name, String alias, String language, String synopsis,
					  String rules, String joinkey, int hide_mode)
      throws DataException, AccessError
  {
    env.testPermission(EnvUser.PERM_CREATECOMMUNITY,"You are not authorized to create new communities.");

    // Convert the "hide mode" value into the two hide flags.
    boolean hide_dir = (hide_mode!=CommunityContext.HIDE_NONE);
    boolean hide_search = (hide_mode==CommunityContext.HIDE_BOTH);

    // Create the new community's database entries and internal data.
    CommunityData new_comm = CommunityCoreData.createCommunity(env,name,alias,uid,language,synopsis,rules,
							       joinkey,hide_dir,hide_search);

    // Create the community context we return to the user.
    CommunityContext rc = new CommunityUserContextImpl(env,new_comm);

    // And that's it!  You expected lightning bolts maybe? :-)

    return rc;

  } // end createCommunity

  public boolean canCreateCommunity()
  {
    return env.testPermission(EnvUser.PERM_CREATECOMMUNITY);

  } // end canCreateCommunity

  public List getSideBoxList() throws DataException
  {
    Connection conn = null;
    ArrayList rc = new ArrayList();

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();

      // retrieve the necessary rows from the sideboxes table
      ResultSet rs = stmt.executeQuery("SELECT boxid, sequence FROM sideboxes WHERE uid = " + uid
				       + " ORDER BY sequence;");
      while (rs.next())
      { // create the implementation objects and return them all
	SideBoxDescriptor sbd =
            new SideBoxDescriptorImpl(env,env.getEngine().getMasterSideBoxDescriptor(rs.getInt(1)),
				      rs.getInt(2));
	rc.add(sbd);

      } // end while

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error getting side box config: " + e.getMessage(),e);
      throw new DataException("unable to retrieve side box config: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

    return Collections.unmodifiableList(rc);

  } // end getSideBoxList

  public void addSideBox(int id) throws DataException
  {
    if (env.getEngine().getMasterSideBoxDescriptor(id)==null)
      throw new DataException("invalid sidebox ID: " + id);

    Connection conn = null;

    try
    { // retrieve a connection
      conn = env.getConnection();
      Statement stmt = conn.createStatement();

      stmt.executeUpdate("LOCK TABLES sideboxes WRITE;");
      try
      { // do a quickie query to see if we're already in the sidebox list
	StringBuffer sql = new StringBuffer("SELECT sequence FROM sideboxes WHERE uid = ");
	sql.append(uid).append(" AND boxid = ").append(id).append(';');
	ResultSet rs = stmt.executeQuery(sql.toString());
	if (rs.next())
	  return;  // already in sidebox list - this is a no-op

	// find a sequence number for the new entry
	sql.setLength(0);
	sql.append("SELECT MAX(sequence) FROM sideboxes WHERE uid = ").append(uid).append(';');
	rs = stmt.executeQuery(sql.toString());
	if (!(rs.next()))
	  throw new InternalStateError("bogus query result on addSideBox");
	int new_sequence = rs.getInt(1) + 100;

	// add the new record
	sql.setLength(0);
	sql.append("INSERT INTO sideboxes (uid, sequence, boxid) VALUES (").append(uid).append(", ");
	sql.append(new_sequence).append(", ").append(id).append(");");
	stmt.executeUpdate(sql.toString());

      } // end try
      finally
      { // make sure the table is unlocked before we go
	Statement ulk_stmt = conn.createStatement();
	ulk_stmt.executeUpdate("UNLOCK TABLES;");

      } // end finally

    } // end try
    catch (SQLException e)
    { // this becomes a DataException
      logger.error("DB error adding to sidebox list: " + e.getMessage(),e);
      throw new DataException("error adding to sidebox list: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      env.releaseConnection(conn);

    } // end finally

  } // end addSideBox

  public List getConferenceHotlist() throws DataException
  {
    return ConferenceUserContextImpl.getUserHotlist(env);

  } // end getConferenceHotlist

  public boolean hasAdminAccess()
  {
    return env.testPermission(EnvUser.PERM_SYSADMINACCESS);

  } // end hasAdminAccess

  public AdminOperations getAdminInterface() throws AccessError
  {
    env.testPermission(EnvUser.PERM_SYSADMINACCESS,"You are not permitted to administer the server.");
    return new AdminOperationsImpl(env);

  } // end getAdminInterface

  public Locale getLocale() throws DataException
  {
    if (my_locale==null)
      loadPrefs();

    return my_locale;

  } // end getLocale

  public void setLocale(Locale locale) throws DataException
  {
    Connection conn = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();

      // create the update statement
      StringBuffer sql = new StringBuffer("UPDATE userprefs SET localeid = '");
      sql.append(SQLUtil.encodeString(locale.toString())).append("' WHERE uid = ").append(uid).append(';');

      // execute the statement
      stmt.executeUpdate(sql.toString());

      // replace the locale here
      my_locale = locale;

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting user locale: " + e.getMessage(),e);
      throw new DataException("unable to set user locale: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end setLocale

  public TimeZone getTimeZone() throws DataException
  {
    if (my_tz==null)
      loadPrefs();

    return my_tz;

  } // end getTimeZone

  public void setTimeZone(TimeZone timezone) throws DataException
  {
    Connection conn = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();

      // create the update statement
      StringBuffer sql = new StringBuffer("UPDATE userprefs SET tzid = '");
      sql.append(SQLUtil.encodeString(timezone.getID())).append("' WHERE uid = ").append(uid).append(';');

      // execute the statement
      stmt.executeUpdate(sql.toString());

      // replace the locale here
      my_tz = timezone;

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting user timezone: " + e.getMessage(),e);
      throw new DataException("unable to set user timezone: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end setTimeZone

  public String getAuthenticationToken() throws AccessError, DataException
  {
    if (!isLoggedIn())
    { // can't generate an authentication token if we're not authenticated!
      logger.error("UserContext not authenticated, cannot generate auth token");
      throw new AccessError("You cannot generate an authentication token without logging in.");

    } // end if

    // Generate a random authentication string and poke it into the database for this user.
    String tokenauth = env.getEngine().generateRandomAuthString();
    Connection conn = null;

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE users SET tokenauth = '");
      sql.append(tokenauth).append("' WHERE uid = ").append(uid).append(';');
      stmt.executeUpdate(sql.toString());

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting token authentication string: " + e.getMessage(),e);
      throw new DataException("Unable to set authentication token: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

    // Build the full authentication token string value.
    int checkvalue = uid ^ tokenauth.hashCode();
    StringBuffer buf = new StringBuffer(AUTH_TOKEN_PREFIX);
    buf.append(uid).append(AUTH_TOKEN_SEP).append(tokenauth).append(AUTH_TOKEN_SEP).append(checkvalue);
    buf.append(AUTH_TOKEN_SEP);
    return buf.toString();

  } // end getAuthenticationToken

  public boolean authenticateWithToken(String token) throws DataException
  {
    if (isLoggedIn())
    { // already authenticated, can't authenticate again
      logger.error("UserContext already authenticated (with uid " + uid + ")");
      throw new InternalStateError("context already authenticated");

    } // end if

    if (logger.isDebugEnabled())
      logger.debug("decoding authtoken: " + token);

    // Pick apart the authentication token value.
    if (!(token.startsWith(AUTH_TOKEN_PREFIX)))
    { // token parse error
      logger.error("Token parse error: prefix not valid");
      return false;

    } // end if

    int xstart = AUTH_TOKEN_PREFIX.length();
    int xend = token.indexOf(AUTH_TOKEN_SEP,xstart);
    if (xend<0)
    { // could not find the UID separator
      logger.error("Token parse error: UID sep not found");
      return false;

    } // end if

    int pending_uid;
    try
    { // get the user ID
      pending_uid = Integer.parseInt(token.substring(xstart,xend));

    } // end try
    catch (NumberFormatException nfe)
    { // we couldn't parse the UID
      logger.error("Token parse error: invalid UID value");
      return false;

    } // end catch

    xstart = xend + 1;
    xend = token.indexOf(AUTH_TOKEN_SEP,xstart);
    if (xend<0)
    { // could not find the auth string separator
      logger.error("Token parse error: auth string sep not found");
      return false;

    } // end if

    String pending_auth = token.substring(xstart,xend);
    if (!(env.getEngine().isValidRandomAuthString(pending_auth)))
    { // the auth string is not valid by the rules under which it was generated
      logger.error("Token parse error: invalid auth string value");
      return false;

    } // end if

    xstart = xend + 1;
    xend = token.indexOf(AUTH_TOKEN_SEP,xstart);
    if (xend<0)
    { // could not find the checkvalue separator
      logger.error("Token parse error: checkvalue sep not found");
      return false;

    } // end if

    int checkvalue;
    try
    { // get the check value
      checkvalue = Integer.parseInt(token.substring(xstart,xend));

    } // end try
    catch (NumberFormatException nfe)
    { // we couldn't parse the checkvalue
      logger.error("Token parse error: invalid checkvalue");
      return false;

    } // end catch

    if (checkvalue!=(pending_uid ^ pending_auth.hashCode()))
    { // the checkvalue does not match what it should - possible corrupted token
      logger.error("Token parse error: checkvalue does not match");
      return false;

    } // end if

    // At this point, we now have a UID and authentication string extracted from the token.
    // Proceed to authenticate.
    if (logger.isDebugEnabled())
      logger.debug("Authenticating user ID#" + pending_uid);

    Connection conn = null;
    AuditRecord ar = null;
      
    try
    { // look for a user record matching this user ID
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery("SELECT * FROM users WHERE uid = " + pending_uid + ";");

      if (!(rs.next()))
      {  // user not found
	logger.error("...user not found");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,0,remote_addr,"Bad token UID: " + pending_uid);
	return false;

      } // end if

      if (rs.getBoolean("is_anon"))
      { // can't log in as Anonymous Honyak
	logger.error("...user is the Anonymous Honyak, can't explicitly login");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,pending_uid,remote_addr,"Anonymous user");
	return false;

      } // end if

      if (rs.getBoolean("lockout"))
      { // account locked out
	logger.error("...user is locked out by the Admin");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,pending_uid,remote_addr,"Account locked out");
	return false;

      } // end if

      // compare the stored token auth value to what we have
      if (!(pending_auth.equals(rs.getString("tokenauth"))))
      { // the auth string is bad - we can't log in
	logger.warn("...invalid authentication string");
	ar = new AuditRecord(AuditRecord.LOGIN_FAIL,pending_uid,remote_addr,"Bad auth-string");
	return false;

      } // end if

      if (logger.isDebugEnabled())
	logger.debug("...authenticated");

      // we're authenticated - load the user data into the context
      loadUserData(rs);

      // update the "last access" time in the database
      java.util.Date mydate = new java.util.Date();
      stmt.executeUpdate("UPDATE users SET lastaccess = '" + SQLUtil.encodeDate(mydate)
			 + "' WHERE uid = " + uid + ";");

      // update the "last access" time in this object
      last_access = mydate;

      // an audit record indicating we logged in OK
      ar = new AuditRecord(AuditRecord.LOGIN_OK,uid,remote_addr);

      if (logger.isDebugEnabled())
	logger.debug("...context loaded, we're ready :-)");

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error reading user data: " + e.getMessage(),e);
      throw new DataException("unable to access user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end if

    return true;  // token authentication worked!

  } // end authenticateWithToken

  public Advertisement selectAd()
  {
    // just get a random ad for now
    return AdvertisementImpl.getRandomAd(env);

  } // end selectAd

  public boolean displayPostPictures() throws DataException
  {
    if (flags==null)
      loadPrefs();

    return flags.get(BF_POSTPICTURES);

  } // end if

  public UserProperties getProperties() throws DataException
  {
    if (flags==null)
      loadPrefs();
    return createProperties();

  } // end getProperties

  public void setProperties(UserProperties props) throws DataException
  {
    if (flags==null)
      loadPrefs();
    UserProperties orig = createProperties();
    BitSet delta = storeProperties(props);
    if (delta.length()==0)
      return;

    boolean succeeded = false;
    try
    { // update the properties in the database
      updateProperties(delta);
      succeeded = true;

    } // end try
    finally
    { // make sure the connection is released before we go
      if (!succeeded)
	storeProperties(orig);

    } // end finally

  } // end setProperties

  /*--------------------------------------------------------------------------------
   * Implementations from interface UserBackend
   *--------------------------------------------------------------------------------
   */

  public int realUID()
  {
    return uid;

  } // end realUID

  public int realBaseLevel()
  {
    return level;

  } // end realBaseLevel

  public String userRemoteAddress()
  {
    return remote_addr;

  } // end userRemoteAddress

  public String userDefaultPseud() throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("userDefaultPseud() for UID " + uid);
    if (my_pseud==null)
      getContactInfo();
    return my_pseud;

  } // end userDefaultPseud

  public boolean userIsAnonymous()
  {
    return is_anon;

  } // end userIsAnonymous

  public String realUserName()
  {
    return username;

  } // end realUserName

  public String realEmailAddress() throws DataException
  {
    if (my_email==null)
      getContactInfo();
    return my_email;

  } // end realEmailAddress

  public String realFullName() throws DataException
  {
    if (full_name==null)
      getContactInfo();
    return full_name;

  } // end realFullName

  public void saveMRU(String tag, Object data)
  {
    mru_cache.put(tag,data);

  } // end saveMRU

  /*--------------------------------------------------------------------------------
   * Operations private to implementation package
   *--------------------------------------------------------------------------------
   */

  void loadAnonymous(String remote_addr) throws DataException
  {
    Connection conn = null;
    if (logger.isDebugEnabled())
      logger.debug("loadAnonymous() on UserContext: addr " + remote_addr);

    try
    { // retrieve a connection from the data pool
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery("SELECT * FROM users WHERE is_anon = 1;");
      if (!(rs.next()))
      { // could not find the anonymous user - this is an internal error
	logger.error("WTF? anonymous user not found?!?");
	throw new DataException("INTERNAL ERROR - cannot find anonymous user");

      } // end if

      loadUserData(rs);  // load the data for Anonymous Honyak

      this.remote_addr = remote_addr;

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("error loading anonymous user data: " + e.getMessage(),e);
      throw new DataException("unable to access user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end loadAnonymous

  void loadNewUser(String remote_addr, int uid, int level, String username, int confirm_num,
		   java.util.Date created, java.util.Date last_access)
  {
    if (logger.isDebugEnabled())
      logger.debug("loadNewUser() on UserContext: addr " + remote_addr + ", uid " + uid + ", level "
		   + level + ", username \"" + username + "\"");

    this.remote_addr = remote_addr;
    this.uid = uid;
    this.contactid = -1;
    this.level = level;
    this.username = username;
    this.confirm_num = confirm_num;
    this.created = created;
    this.last_access = last_access;

  } // end loadNewUser

  void autoJoinCommunities() throws DataException
  {
    Connection conn = null;

    try
    { // get a database connection and call the internal function
      conn = env.getConnection();
      autoJoinCommunities(conn);

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("error autojoining communities: " + e.getMessage(),e);
      throw new DataException("unable to autojoin communities: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end autoJoinCommunities
  
  /*--------------------------------------------------------------------------------
   * Static initializer
   *--------------------------------------------------------------------------------
   */

  static
  { // initialize the "all properties" bit set
    BitSet tmp = new BitSet();
    tmp.set(PROP_FLAGS);
    ALL_PROPS = tmp;

  } // end static initializer

} // end class UserContextImpl
