/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.awt.Dimension;
import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
import org.w3c.dom.*;
import com.silverwrist.util.*;
import com.silverwrist.util.cache.*;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.core.internals.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.htmlcheck.*;
import com.silverwrist.venice.htmlcheck.dict.*;
import com.silverwrist.venice.htmlcheck.filters.*;
import com.silverwrist.venice.security.*;

public class VeniceEngineImpl implements VeniceEngine, EngineBackend
{
  /*--------------------------------------------------------------------------------
   * Internal class storing feature information.
   *--------------------------------------------------------------------------------
   */

  class VeniceFeatureDef implements CommunityFeature
  {
    int code;
    String symbol;
    String name;
    String applet;
    boolean is_default;
    boolean is_locked;
    boolean is_hidden;
    boolean requires_read;
    int sequence;
    int min_level;
    
    VeniceFeatureDef(int code, ResultSet rs) throws SQLException
    {
      this.code = code;
      this.is_default = rs.getBoolean("is_default");
      this.is_locked = rs.getBoolean("is_locked");
      this.is_hidden = rs.getBoolean("is_hidden");
      this.requires_read = rs.getBoolean("require_read");
      this.sequence = rs.getInt("sequence");
      this.min_level = rs.getInt("min_level");
      this.symbol = rs.getString("symbol");
      this.name = rs.getString("name");
      this.applet = rs.getString("applet");

    } // end constructor

    public int getFeatureCode()
    {
      return code;

    } // end getFeatureCode

    public String getSymbol()
    {
      return symbol;

    } // end getSymbol

    public String getName()
    {
      return name;

    } // end getName

    public String getApplet()
    {
      return applet;

    } // end getApplet

    public int getSequence()
    {
      return sequence;

    } // end getSequence

    boolean featureAllowed(int level, boolean has_read)
    {
      if (level<min_level)
	return false;
      if (requires_read)
	return has_read;
      else
	return true;

    } // end featureAllowed

    boolean isVisible()
    {
      return !is_hidden;

    } // end isVisible

    boolean isLocked()
    {
      return is_locked;

    } // end isLocked

    boolean isDefault()
    {
      return is_default;

    } // end isDefault

  } // end class VeniceFeatureDef

  /*--------------------------------------------------------------------------------
   * Internal class storing side box information.
   *--------------------------------------------------------------------------------
   */

  class MasterSideBox implements SideBoxDescriptor
  {
    private int id;
    private String title;
    private String anon_title;
    private String classname;

    MasterSideBox(Element section) throws ConfigException
    {
      try
      { // get the ID value for this sidebox
	String tmp = section.getAttribute("id");
	if (StringUtil.isStringEmpty(tmp))
	{ // there's no 
	  logger.fatal("<sidebox/> specified with no ID!");
	  throw new ConfigException("no ID specified in <sidebox/>",section);

	} // end if

	id = Integer.parseInt(tmp);

      } // end try
      catch (NumberFormatException e)
      { // sidebox ID not numeric
	logger.fatal("<sidebox/> ID not numeric!");
	throw new ConfigException("non-numeric ID specified in <sidebox/>",section);

      } // end catch

      DOMElementHelper section_h = new DOMElementHelper(section);
      title = section_h.getSubElementText("title");
      anon_title = section_h.getSubElementText("anon-title");
      classname = section_h.getSubElementText("factory-class");

      if (StringUtil.isStringEmpty(title) || StringUtil.isStringEmpty(classname))
      { // the configuration is not complete
	logger.fatal("<sidebox/> config incomplete (missing <title/> or <factory-class/>)!");
	throw new ConfigException("configuration of <sidebox/> is not complete!",section);

      } // end if

      try
      { // try to resolve the classname here so we can be sure it's cool
	Class.forName(classname);

      } // end try
      catch (ClassNotFoundException e)
      { // the class can't be resolved - we are h0sed
	logger.fatal("<sidebox/> config <factory-class/> not valid!");
	throw new ConfigException("<factory-class/> in <sidebox/> is not a valid class!",section);

      } // end catch

      if (StringUtil.isStringEmpty(anon_title))
	anon_title = title;  // just set as the same in the event of difficulty

    } // end constructor

    public int getID()
    {
      return id;

    } // end getID

    public int getSequence()
    {
      return -1;

    } // end getSequence

    public String getTitle(boolean anonymous)
    {
      return (anonymous ? anon_title : title);

    } // end getTitle

    public String getFactoryClassName()
    {
      return classname;

    } // end getClassname

  } // end class MasterSideBox

  /*--------------------------------------------------------------------------------
   * Internal class for returning side box information.
   *--------------------------------------------------------------------------------
   */

  static class MasterSideBoxList extends AbstractList
  {
    private MasterSideBox[] array;

    MasterSideBoxList(MasterSideBox[] array)
    {
      this.array = array;

    } // end constructor

    public Object get(int index)
    {
      return array[index];
      
    } // end get

    public int size()
    {
      return array.length;

    } // end size

  } // end class MasterSideBoxList

  /*--------------------------------------------------------------------------------
   * Internal class for creating new CommunityCoreData objects.
   *--------------------------------------------------------------------------------
   */

  protected class CommunityCoreDataCreator implements ObjectFactory
  {
    protected CommunityCoreDataCreator()
    { // do nothing
    } // end constructor

    public Object newObject(Object key) throws ObjectFactoryException
    {
      Integer xcid = (Integer)key;
      try
      { // create the desired object
	return new CommunityCoreData(env,xcid.intValue());

      } // end try
      catch (DataException e)
      { // rethrow as a "wrapped" exception
	throw new ObjectFactoryException(e);

      } // end catch

    } // end build

  } // end class CommunityCoreDataCreator

  /*--------------------------------------------------------------------------------
   * Internal class for creating new ConferenceCoreData objects.
   *--------------------------------------------------------------------------------
   */

  protected class ConferenceCoreDataCreator implements ObjectFactory
  {
    protected ConferenceCoreDataCreator()
    { // do nothing
    } // end constructor

    public Object newObject(Object key) throws ObjectFactoryException
    {
      Integer xconf = (Integer)key;
      try
      { // create the desired object
	return new ConferenceCoreData(env,xconf.intValue());

      } // end try
      catch (DataException e)
      { // rethrow as a "wrapped" exception
	throw new ObjectFactoryException(e);

      } // end catch

    } // end build

  } // end class ConferenceCoreDataCreator

  /*--------------------------------------------------------------------------------
   * Internal class for password change requests
   *--------------------------------------------------------------------------------
   */

  class PasswordChangeRequest
  {
    private static final long EXPIRE_TIME = 3600000L;  // one hour

    private int uid;                    // UID of the password change request
    private String username;            // username of the password change request
    private int authentication;         // authentication value
    private String email;               // email address
    private long timestamp;             // current timestamp

    PasswordChangeRequest(int uid, String username, String email)
    {
      this.uid = uid;
      this.username = username;
      this.email = email;
      this.authentication = rng.nextInt(Integer.MAX_VALUE);
      this.timestamp = System.currentTimeMillis();

    } // end constructor

    final int getUID()
    {
      return uid;

    } // end getUID

    final String getUserName()
    {
      return username;

    } // end getUserName

    final int getAuthentication()
    {
      return authentication;

    } // end getAuthentication

    final String getEmail()
    {
      return email;

    } // end getEmail

    final boolean isExpired()
    {
      long diff = System.currentTimeMillis() - timestamp;
      return (diff>EXPIRE_TIME);

    } // end isExpired
    
  } // end class PasswordChangeRequest

  /*--------------------------------------------------------------------------------
   * Static data values
   *--------------------------------------------------------------------------------
   */

  // Property indices
  private static final int PROP_FLAGS = 0;       // flags

  private static final int _PPROP_IP_BASE = 256; // base for "pseudo property" indexes

  private static final BitSet IP_PROPS;          // BitSet representing the "integer parameter" properties
  private static final BitSet ALL_PROPS;         // BitSet representing all properties

  private static Category logger = Category.getInstance(VeniceEngineImpl.class);

  private static final String AUTH_ALPHABET =
      "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz./";
  private static final int AUTH_STRING_LEN = 32;

  private static final Dimension DEFAULT_DIM_USERPHOTO = new Dimension(100,100);
  private static final Dimension DEFAULT_DIM_COMMUNITYLOGO = new Dimension(110,65);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private Document config = null;                     // configuration data
  private EnvEngine env = null;                       // my environment
  private Random rng;                                 // random number generator
  private Properties email_props = null;              // email properties
  private javax.mail.Session mailsession = null;      // email session object
  private Hashtable stock_messages = null;            // stock messages holder
  private ObjectCache comm_objcache = new ObjectCache(new CommunityCoreDataCreator());
  private VeniceFeatureDef[] features;                // master feature table
  private Hashtable feature_syms = new Hashtable();   // hashtable mapping symbols to features
  private ObjectCache conf_objcache = new ObjectCache(new ConferenceCoreDataCreator());
  private HTMLCheckerConfig[] html_configs;           // holder for HTML checker configurations
  private int[] gp_ints;                              // global integer parameters
  private MasterSideBox[] sideboxes;                  // master sidebox table
  private Hashtable sidebox_ids = new Hashtable();    // maps sidebox IDs to MasterSideBox objects
  private LinkedList cache_fp_posts = new LinkedList();  // all posts that have been published to front page
  private boolean cache_fp_posts_busy = false;        // busy flag for above vector
  private HashSet no_compress_types = new HashSet();  // the file types that can't be compressed
  private HashMap password_changes = new HashMap();   // current password change requests
  private OptionSet global_flags = new OptionSet();   // global option flags
  private SecurityMonitor global_security;            // the global security monitor
  private SecurityMonitor community_security;         // the community security monitor

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public VeniceEngineImpl()
  {
    // Initialize the random number generator.
    java.util.Date seed_date = new java.util.Date();
    rng = new Random(seed_date.getTime());

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private void checkInitialized()
  {
    if (config==null)
      throw new InternalStateError("Venice engine not initialized!");

  } // end checkInitialized

  private void loadDefaults(Statement stmt) throws SQLException, DataException
  {
    final String query1 = "SELECT ndx, data FROM propglobal;";
    ResultSet rs = stmt.executeQuery(query1);
    while (rs.next())
    { // load the property values based on indexes
      switch (rs.getInt(1))
      {
	case PROP_FLAGS:
	  global_flags.assign(rs.getString(2));
	  break;

	default:
	  break;  // don't understand property index - ignored

      } // end switch

    } // end while

    final String query2 =
        "SELECT posts_per_page, old_posts_at_top, max_search_page, max_sig_mbr_page, max_conf_mbr_page, "
      + "fp_posts, num_audit_page, sig_create_lvl FROM globals;";
    rs = stmt.executeQuery(query2);
    if (!(rs.next()))
      throw new DataException("Globals table does not appear to be loaded!");

    // stash the globals from the database
    gp_ints[IP_POSTSPERPAGE] = rs.getInt(1);
    gp_ints[IP_POSTSATTOP] = rs.getInt(2);
    gp_ints[IP_MAXSEARCHRETURN] = rs.getInt(3);
    gp_ints[IP_MAXCOMMUNITYMEMBERDISPLAY] = rs.getInt(4);
    gp_ints[IP_MAXCONFMEMBERDISPLAY] = rs.getInt(5);
    gp_ints[IP_NUMFRONTPAGEPOSTS] = rs.getInt(6);
    gp_ints[IP_NUMAUDITRECSPERPAGE] = rs.getInt(7);
    gp_ints[IP_CREATECOMMUNITYLVL] = rs.getInt(8);
    
  } // end loadDefaults

  private synchronized void updateDefaults(BitSet delta) throws DataException
  {
    Connection conn = null;
    try
    { // get a database connection
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer();

      if (delta.get(PROP_FLAGS))
      { // update the global flags
	sql.append("UPDATE propglobal SET data = '").append(global_flags.asString()).append("' WHERE ndx = ");
	sql.append(PROP_FLAGS).append(";");
	stmt.executeUpdate(sql.toString());
	sql.setLength(0);

      } // end if

      BitSet tmp = (BitSet)(delta.clone());
      tmp.and(IP_PROPS);
      if (tmp.length()>0)
      { // need to update the globals table
	sql.append("UPDATE globals SET");
	if (delta.get(_PPROP_IP_BASE + IP_POSTSPERPAGE))
	  sql.append(" posts_per_page = ").append(gp_ints[IP_POSTSPERPAGE]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_POSTSATTOP))
	  sql.append(" old_posts_at_top = ").append(gp_ints[IP_POSTSATTOP]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_MAXSEARCHRETURN))
	  sql.append(" max_search_page = ").append(gp_ints[IP_MAXSEARCHRETURN]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_MAXCOMMUNITYMEMBERDISPLAY))
	  sql.append(" max_sig_mbr_page = ").append(gp_ints[IP_MAXCOMMUNITYMEMBERDISPLAY]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_MAXCONFMEMBERDISPLAY))
	  sql.append(" max_conf_mbr_page = ").append(gp_ints[IP_MAXCONFMEMBERDISPLAY]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_NUMFRONTPAGEPOSTS))
	  sql.append(" fp_posts = ").append(gp_ints[IP_NUMFRONTPAGEPOSTS]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_NUMAUDITRECSPERPAGE))
	  sql.append(" num_audit_page = ").append(gp_ints[IP_NUMAUDITRECSPERPAGE]).append(',');
	if (delta.get(_PPROP_IP_BASE + IP_CREATECOMMUNITYLVL))
	  sql.append(" sig_create_lvl = ").append(gp_ints[IP_CREATECOMMUNITYLVL]).append(',');
	sql.setLength(sql.length()-1);  // trim off the last comma
	sql.append(';');
	stmt.executeUpdate(sql.toString());
	sql.setLength(0);

      } // end if

    } // end try
    catch (SQLException e)
    { // convert exceptions to DataException
      logger.fatal("Database error storing defaults: " + e.getMessage(),e);
      throw new DataException("error saving new defaults: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      env.releaseConnection(conn);

    } // end finally

  } // end updateDefaults

  private synchronized void updateDefaults() throws DataException
  {
    updateDefaults(ALL_PROPS);

  } // end updateDefaults

  private final GlobalProperties createProperties()
  {
    GlobalProperties rc = new GlobalProperties();
    rc.setPostsPerPage(gp_ints[IP_POSTSPERPAGE]);
    rc.setOldPostsAtTop(gp_ints[IP_POSTSATTOP]);
    rc.setSearchItemsPerPage(gp_ints[IP_MAXSEARCHRETURN]);
    rc.setCommunityMembersPerPage(gp_ints[IP_MAXCOMMUNITYMEMBERDISPLAY]);
    rc.setConferenceMembersPerPage(gp_ints[IP_MAXCONFMEMBERDISPLAY]);
    rc.setPostsOnFrontPage(gp_ints[IP_NUMFRONTPAGEPOSTS]);
    rc.setAuditRecordsPerPage(gp_ints[IP_NUMAUDITRECSPERPAGE]);
    rc.setCommunityCreateLevel(gp_ints[IP_CREATECOMMUNITYLVL]);
    rc.setDisplayPostPictures(global_flags.get(BP_POSTPICTURES));
    return rc;

  } // end createProperties

  private final BitSet storeProperties(GlobalProperties props)
  {
    BitSet rc = new BitSet();
    if (global_flags.assign(BP_POSTPICTURES,props.getDisplayPostPictures()))
      rc.set(PROP_FLAGS);
    if (gp_ints[IP_POSTSPERPAGE]!=props.getPostsPerPage())
    { // record variable change
      gp_ints[IP_POSTSPERPAGE] = props.getPostsPerPage();
      rc.set(_PPROP_IP_BASE + IP_POSTSPERPAGE);

    } // end if

    if (gp_ints[IP_POSTSATTOP]!=props.getOldPostsAtTop())
    { // record variable change
      gp_ints[IP_POSTSATTOP] = props.getOldPostsAtTop();
      rc.set(_PPROP_IP_BASE + IP_POSTSATTOP);

    } // end if

    if (gp_ints[IP_MAXSEARCHRETURN]!=props.getSearchItemsPerPage())
    { // record variable change
      gp_ints[IP_MAXSEARCHRETURN] = props.getSearchItemsPerPage();
      rc.set(_PPROP_IP_BASE + IP_MAXSEARCHRETURN);

    } // end if

    if (gp_ints[IP_MAXCOMMUNITYMEMBERDISPLAY]!=props.getCommunityMembersPerPage())
    { // record variable change
      gp_ints[IP_MAXCOMMUNITYMEMBERDISPLAY] = props.getCommunityMembersPerPage();
      rc.set(_PPROP_IP_BASE + IP_MAXCOMMUNITYMEMBERDISPLAY);

    } // end if

    if (gp_ints[IP_MAXCONFMEMBERDISPLAY]!=props.getConferenceMembersPerPage())
    { // record variable change
      gp_ints[IP_MAXCONFMEMBERDISPLAY] = props.getConferenceMembersPerPage();
      rc.set(_PPROP_IP_BASE + IP_MAXCONFMEMBERDISPLAY);

    } // end if

    if (gp_ints[IP_NUMFRONTPAGEPOSTS]!=props.getPostsOnFrontPage())
    { // record variable change
      gp_ints[IP_NUMFRONTPAGEPOSTS] = props.getPostsOnFrontPage();
      rc.set(_PPROP_IP_BASE + IP_NUMFRONTPAGEPOSTS);

    } // end if

    if (gp_ints[IP_NUMAUDITRECSPERPAGE]!=props.getAuditRecordsPerPage())
    { // record variable change
      gp_ints[IP_NUMAUDITRECSPERPAGE] = props.getAuditRecordsPerPage();
      rc.set(_PPROP_IP_BASE + IP_NUMAUDITRECSPERPAGE);

    } // end if

    if (gp_ints[IP_CREATECOMMUNITYLVL]!=props.getCommunityCreateLevel())
    { // record variable change
      gp_ints[IP_CREATECOMMUNITYLVL] = props.getCommunityCreateLevel();
      rc.set(_PPROP_IP_BASE + IP_CREATECOMMUNITYLVL);

    } // end if

    return rc;

  } // end storeProperties

  /*--------------------------------------------------------------------------------
   * Implementations from interface VeniceEngine
   *--------------------------------------------------------------------------------
   */

  /**
   * Loads the Venice engine configuration from the named XML file and creates appropriate subobjects,
   * such as the data pool.
   *
   * @param config The XML document containing the configuration information for the engine.
   * @param app_root The root directory of the Web application, which ends with a "/".
   * @exception com.silverwrist.venice.core.ConfigException An error occurred during engine configuration;
   *                                                        the engine could not be initialized.
   * @exception com.silverwrist.venice.core.DataException The Venice engine data could not be loaded.
   */
  public void initialize(Document config, String app_root) throws ConfigException, DataException
  {
    int i;  // loop counter

    if (this.config!=null)
    { // already configured!
      logger.error("Venice engine already initialized");
      throw new ConfigException("Venice engine already initialized");

    } // end if

    this.config = config;

    DataPool datapool = null;
    ArrayList dictionary_tmp;
    try
    { // first, verify that this is a valid configuration
      Element root = config.getDocumentElement();
      if (!(root.getTagName().equals("venice-config")))
      { // not the correct root tag name
	logger.fatal("config document is not a <venice-config/> document (root tag: <"
		     + root.getTagName() + "/>)");
	throw new ConfigException("document is not a <venice-config/> document",root);

      } // end if

      DOMElementHelper root_h = new DOMElementHelper(root);

      // Get the <engine/> section.
      Element engine_sect = root_h.getSubElement("engine");
      if (engine_sect==null)
      { // unable to find the engine section
	logger.fatal("config document has no <engine/> section");
	throw new ConfigException("no <engine/> section found in config file",root);

      } // end if

      DOMElementHelper engine_h = new DOMElementHelper(engine_sect);

      // Get the name of the sidebox config file.
      String sidebox_config = engine_h.getSubElementText("sidebox-config");
      if (sidebox_config==null)
      { // unable to find the sidebox config file
	logger.fatal("<engine/> section has no <sidebox-config/> element");
	throw new ConfigException("no <sidebox-config/> element found in <engine/> section",engine_sect);

      } // end if

      if (!(sidebox_config.startsWith("/")))
	sidebox_config = app_root + sidebox_config;
      Document sidebox_cfg_file = Startup.loadConfiguration(sidebox_config);
      Element sb_root = sidebox_cfg_file.getDocumentElement();
      if (!(sb_root.getTagName().equals("sidebox-config")))
      { // the sidebox configuration file isn't the right type
	logger.fatal("config document is not a <sidebox-config/> document (root tag: <"
		     + sb_root.getTagName() + "/>)");
	throw new ConfigException("document is not a <sidebox-config/> document",sb_root);

      } // end if

      NodeList sb_nodes = sb_root.getChildNodes();
      ArrayList sidebox_tmp = new ArrayList();
      for (i=0; i<sb_nodes.getLength(); i++)
      { // extract each sidebox section from the list and build a new master sidebox object
	Node s = sb_nodes.item(i);
	if ((s.getNodeType()==Node.ELEMENT_NODE) && (s.getNodeName().equals("sidebox")))
	  sidebox_tmp.add(new MasterSideBox((Element)s));

      } // end for

      // store the real master sidebox table as an array
      sideboxes = (MasterSideBox[])(sidebox_tmp.toArray(new MasterSideBox[0]));
      if (logger.isDebugEnabled())
	logger.debug(sideboxes.length + " sidebox definitions loaded from XML");

      // Get the <database/> section.
      Element db_sect = root_h.getSubElement("database");
      if (db_sect==null)
      { // unable to find the database section
	logger.fatal("config document has no <database/> section");
	throw new ConfigException("no <database/> section found in config file",root);

      } // end if

      try
      { // allocate the data pool object
	datapool = new DataPool(db_sect);

      } // end try
      catch (SQLException e)
      { // could not initialize the SQL database!  this is a big fat error!
	logger.fatal("data pool init failed - " + e.getMessage(),e);
	throw new ConfigException("database access initialization failed - " + e.getMessage(),e);

      } // end catch

      Element security_sect = root_h.getSubElement("security");
      if (security_sect==null)
      { // no "security" section...bad!
	logger.fatal("config document has no <security/> section");
	throw new ConfigException("no <security/> section found in config file",root);

      } // end if

      NodeList sec_nodes = security_sect.getChildNodes();
      for (i=0; i<sec_nodes.getLength(); i++)
      { // scan through and find security monitors to initialize
	Node n = sec_nodes.item(i);
	if ((n.getNodeType()==Node.ELEMENT_NODE) && (n.getNodeName().equals("security-definition")))
	{ // initial security definition
	  SecurityMonitor sm = new StaticSecurityMonitor((Element)n);
	  if (sm.getID().equals("Global"))
	    global_security = sm;
	  else if (sm.getID().equals("Community"))
	    community_security = sm;

	} // end if

      } // end for

      Element email_sect = root_h.getSubElement("email");
      if (email_sect==null)
      { // unable to find the database section
	logger.fatal("config document has no <email/> section");
	throw new ConfigException("no <email/> section found in config file",root);

      } // end if

      // initialize the email properties and get a mail session object
      email_props = new Properties();
      email_props.put("mail.transport.protocol","smtp");
      DOMElementHelper email_sect_h = new DOMElementHelper(email_sect);
      email_props.put("mail.smtp.host",email_sect_h.getSubElementText("smtp-host"));
      email_props.put("mail.from",email_sect_h.getSubElementText("mail-from-addr"));
      email_props.put("com.silverwrist.venice.email.mailer",email_sect_h.getSubElementText("mailer"));
      mailsession = javax.mail.Session.getInstance(email_props,null);

      Element dict_sect = root_h.getSubElement("dictionary");
      if (dict_sect==null)
      { // unable to find the database section
	logger.fatal("config document has no <dictionary/> section");
	throw new ConfigException("no <dictionary/> section found in config file",root);

      } // end if

      // Retrieve the list of dictionary files to load into the spellchecker.
      dictionary_tmp = new ArrayList();
      NodeList dict_nodes = dict_sect.getChildNodes();
      for (i=0; i<dict_nodes.getLength(); i++)
      { // scan the <dictionary> element looking for <file> elements
	Node dn = dict_nodes.item(i);
	if ((dn.getNodeType()==Node.ELEMENT_NODE) && (dn.getNodeName().equals("file")))
	{ // store the file name as a vector element
	  Element del = (Element)dn;
	  DOMElementHelper h = new DOMElementHelper(del);
	  dictionary_tmp.add(h.getElementText());

	} // end if

      } // end for

      Element upload_sect = root_h.getSubElement("upload");
      if (upload_sect==null)
      { // unable to find the "upload" section
	logger.fatal("config document has no <upload/> section");
	throw new ConfigException("no <upload/> section found in config file",root);

      } // end if

      // Look for a "no-compress" blacklist.
      DOMElementHelper upload_sect_h = new DOMElementHelper(upload_sect);
      Element no_compress_sect = upload_sect_h.getSubElement("no-compress");
      if (no_compress_sect!=null)
      { // Initialize the no-compress list.
	NodeList nc_nodes = no_compress_sect.getChildNodes();
	for (i=0; i<nc_nodes.getLength(); i++)
	{ // retrieve all entries from the list
	  Node ncn = nc_nodes.item(i);
	  if (ncn.getNodeType()==Node.ELEMENT_NODE)
	  { // get the element
	    DOMElementHelper ncel = new DOMElementHelper((Element)ncn);
	    if (ncel.getElement().getNodeName().equals("type"))
	    { // add a new element to the list
	      no_compress_types.add(ncel.getElementText().trim().toLowerCase());

	    } // end if
	    else
	    { // this is bad - bail out!
	      logger.fatal("spurious element in <no-compress/> section");
	      throw new ConfigException("<no-compress/> section contains spurious element",no_compress_sect);

	    } // end else

	  } // end if
	  // else skip it

	} // end for

      } // end if
      // else no list, just proceed
      
      Element msg_sect = root_h.getSubElement("messages");
      if (msg_sect==null)
      { // unable to find the database section
	logger.fatal("config document has no <messages/> section");
	throw new ConfigException("no <messages/> section found in config file",root);

      } // end if

      // Initialize the stock messages list.
      stock_messages = new Hashtable();
      NodeList msg_nodes = msg_sect.getChildNodes();
      for (i=0; i<msg_nodes.getLength(); i++)
      { // examine all subnodes to add them to the message text
	Node msgn = msg_nodes.item(i);
	if (msgn.getNodeType()==Node.ELEMENT_NODE)
	{ // add it to the hash table by its tag name
	  Element msgel = (Element)msgn;
	  DOMElementHelper h = new DOMElementHelper(msgel);
	  String txt = h.getElementText();
	  if (txt!=null)
	    stock_messages.put(msgel.getTagName(),txt.trim());

	} // end if

      } // end for

      if (logger.isDebugEnabled())
	logger.debug(stock_messages.size() + " stock messages loaded from config");

    } // end try
    catch (ConfigException ce)
    { // before we leave on a ConfigException, nuke the important data
      this.config = null;
      if (datapool!=null)
	datapool.closeAllConnections();
      throw ce;

    } // end catch

    // Initialize the environment.
    env = new EnvEngine(this,datapool);

    for (i=0; i<sideboxes.length; i++) // insert sideboxes into hashtable
      sidebox_ids.put(new Integer(sideboxes[i].getID()),sideboxes[i]);

    // Allocate the global parameter arrays.
    gp_ints = new int[IPC_NUM_PARAMS];

    // initialize anything that requires us to pull from the database
    Connection conn = null;
    try
    { // get a connection from the data pool
      conn = env.getConnection();

      // load the master feature table
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery("SELECT * FROM refsigftr ORDER BY ftr_code;");
      VeniceFeatureDef[] tmp_array = new VeniceFeatureDef[64];
      int max_value = -1;
      while (rs.next())
      { // store the values as we get them
	int ndx = rs.getInt("ftr_code");
	tmp_array[ndx] = new VeniceFeatureDef(ndx,rs);
	if (ndx>max_value)
	  max_value = ndx;

      } // end while

      // store the real feature table as an array
      features = new VeniceFeatureDef[++max_value];
      System.arraycopy(tmp_array,0,features,0,max_value);
      if (logger.isDebugEnabled())
	logger.debug(max_value + " features loaded from database");

      // load the global defaults
      loadDefaults(stmt);

    } // end try
    catch (SQLException e)
    { // convert exceptions to DataException
      logger.fatal("Database error loading engine data: " + e.getMessage(),e);
      throw new DataException("error initializing Venice engine: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      env.releaseConnection(conn);

    } // end finally

    for (i=0; i<features.length; i++) // insert feature symbols into hashtable
      feature_syms.put(features[i].getSymbol(),features[i]);

    // Here is where we create the HTML Checker and all the goodies it relies on.
    // Start by creating some of the subsidiary objects that get added to HTML Checker configs.
    EmailRewriter email_rewriter = new EmailRewriter();
    HTMLEncodingFilter html_filter = new HTMLEncodingFilter();
    SQLEncodingFilter sql_filter = new SQLEncodingFilter();
    SpellingRewriter spell_rewriter = new SpellingRewriter();
    URLRewriter url_rewriter = new URLRewriter();
    PostLinkRewriter postlink_rewriter = new PostLinkRewriter(env.getDataPool());
    UserNameRewriter username_rewriter = new UserNameRewriter(env.getDataPool());

    // Create the LazyLexicon that holds our dictionary files, and add it to the SpellingRewriter.
    LazyTreeLexicon lex = new LazyTreeLexicon((String[])(dictionary_tmp.toArray(new String[0])));
    spell_rewriter.addDictionary(lex);

    html_configs = new HTMLCheckerConfig[4];  // create the array

    // Create the HTML checker config used to post body text to the database.
    HTMLCheckerConfig cfg = HTMLCheckerCreator.create();
    cfg.setWordWrapLength((short)55);
    cfg.setProcessAngles(true);
    cfg.setProcessParens(true);
    cfg.setDiscardHTMLTags(false);
    cfg.addOutputFilter(html_filter);
    cfg.addOutputFilter(sql_filter);
    cfg.addRawOutputFilter(sql_filter);
    cfg.addStringRewriter(email_rewriter);
    cfg.addStringRewriter(url_rewriter);
    cfg.addTagRewriter(postlink_rewriter);
    cfg.addTagRewriter(username_rewriter);
    cfg.addTagRewriter(email_rewriter);
    cfg.addTagRewriter(url_rewriter);
    cfg.addParenRewriter(username_rewriter);
    cfg.configureNormalTagSet();
    html_configs[HTMLC_POST_BODY] = cfg;

    // Create the HTML checker config used to post pseuds and topic names to the database.
    cfg = HTMLCheckerCreator.create();
    cfg.setWordWrapLength((short)0);
    cfg.setProcessAngles(true);
    cfg.setProcessParens(false);
    cfg.setDiscardHTMLTags(false);
    cfg.addOutputFilter(html_filter);
    cfg.addOutputFilter(sql_filter);
    cfg.addRawOutputFilter(sql_filter);
    cfg.configureRestrictedTagSet();
    html_configs[HTMLC_POST_PSEUD] = cfg;

    // Create the HTML checker config used to preview body text.
    cfg = HTMLCheckerCreator.create();
    cfg.setWordWrapLength((short)55);
    cfg.setProcessAngles(true);
    cfg.setProcessParens(true);
    cfg.setDiscardHTMLTags(false);
    cfg.addOutputFilter(html_filter);
    cfg.addStringRewriter(email_rewriter);
    cfg.addStringRewriter(url_rewriter);
    cfg.addWordRewriter(spell_rewriter);
    cfg.addTagRewriter(postlink_rewriter);
    cfg.addTagRewriter(username_rewriter);
    cfg.addTagRewriter(email_rewriter);
    cfg.addTagRewriter(url_rewriter);
    cfg.addParenRewriter(username_rewriter);
    cfg.configureNormalTagSet();
    html_configs[HTMLC_PREVIEW_BODY] = cfg;
    
    // Create the HTML checker config used to escape body text and pseuds for putting back in the post box.
    cfg = HTMLCheckerCreator.create();
    cfg.setWordWrapLength((short)0);
    cfg.setProcessAngles(false);
    cfg.setProcessParens(false);
    cfg.setDiscardHTMLTags(false);
    cfg.addOutputFilter(html_filter);
    html_configs[HTMLC_ESCAPE_BODY_PSEUD] = cfg;

    if (logger.isDebugEnabled())
      logger.debug("initialize() complete :-)");

  } // end initialize

  public int getNumFeatures()
  {
    checkInitialized();
    return features.length;

  } // end getNumFeatures

  public BitSet getAllFeaturesMask()
  {
    checkInitialized();
    BitSet rc = new BitSet();
    for (int i=0; i<features.length; i++)
      if (features[i].isVisible())
	rc.set(i);

    return rc;

  } // end getAllFeaturesMask

  public UserContext createUserContext(String remote_addr) throws DataException
  {
    checkInitialized();
    UserContextImpl uci = new UserContextImpl(env);
    uci.loadAnonymous(remote_addr);
    if (logger.isDebugEnabled())
      logger.debug("createUserContext(): context loaded :-)");
    return (UserContext)uci;

  } // end createUserContext

  public String getEmailAddressForUser(String username) throws DataException, AccessError
  {
    checkInitialized();
    Connection conn = null;
      
    try
    { // look for a user name matching this user record
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery("SELECT c.email FROM users u, contacts c WHERE u.contactid = "
				       + "c.contactid AND u.username = '" + SQLUtil.encodeString(username)
				       + "';");
      if (!(rs.next()))
      { // could not find email address
	logger.warn("getEmailAddressForUser(\"" + username + "\") => not found!");
	throw new AccessError("Unable to find email address for user \"" + username +
			      "\".  Please contact the system administrator for assistance.");

      } // end if

      String rc = rs.getString(1);
      if (logger.isDebugEnabled())
	logger.debug("getEmailAddressForUser(\"" + username + "\") => \"" + rc + "\"");
      return rc;

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error getting email address for user: " + e.getMessage(),e);
      throw new DataException("unable to access user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end getEmailAddressForUser

  public void sendPasswordReminder(String username) throws DataException, AccessError, EmailException
  {
    checkInitialized();
    Connection conn = null;
      
    try
    { // look for a user name matching this user record
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT c.email, u.uid, u.passreminder FROM users u, contacts c "
					  + "WHERE u.contactid = c.contactid AND u.username = '");
      sql.append(SQLUtil.encodeString(username)).append("';");
      ResultSet rs = stmt.executeQuery(sql.toString());
      if (!(rs.next()))
      { // the email address could not be found
	logger.error("can't find email to send password reminder to \"" + username + "\"");
	throw new AccessError("Unable to find email address for user \"" + username +
			      "\".  Please contact the system administrator for assistance.");

      } // end if

      String email_addr = rs.getString(1);
      if (logger.isDebugEnabled())
	logger.debug("sendPasswordReminder(\"" + username + "\") going out to \"" + email_addr + "\"");

      // Enqueue a password change request.
      PasswordChangeRequest pcr = new PasswordChangeRequest(rs.getInt(2),username,email_addr);
      password_changes.put(new Integer(pcr.getUID()),pcr);

      // Create the message to be sent.
      String message = getStockMessage("reminder");
      if (message==null)
      { // what? the message does not exist?
	logger.error("\"reminder\" message does not exist in config file!");
	throw new DataException("INTERNAL: reminder message not specified");

      } // end if

      // Replace the message variables.
      HashMap vars = new HashMap(4);
      vars.put("username",username);
      vars.put("reminder",rs.getString(3));
      vars.put("change.uid",String.valueOf(pcr.getUID()));
      vars.put("change.auth",String.valueOf(pcr.getAuthentication()));
      message = StringUtil.replaceAllVariables(message,vars);

      // Find the message subject.
      String subject = getStockMessage("reminder-subject");
      if (subject==null)
	subject = "Venice Password Reminder Message";

      // Create the emailer and send the message.
      Emailer emailer = createEmailer();
      emailer.setTo(email_addr);
      emailer.setSubject(subject);
      emailer.setText(message);
      emailer.send();
      if (logger.isDebugEnabled())
	logger.debug("...email sent");

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error getting email address/reminder for user: " + e.getMessage(),e);
      throw new DataException("unable to access user data: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end sendPasswordReminder

  public void completePasswordChange(int uid, int authentication)
      throws DataException, AccessError, EmailException
  {
    checkInitialized();
    Integer key = new Integer(uid);
    PasswordChangeRequest pcr;
    synchronized (this)
    { // make sure we can't pull the same request out twice!
      pcr = (PasswordChangeRequest)(password_changes.get(key));
      if (pcr==null)
	throw new AccessError("Password change request not found.");
      password_changes.remove(key);  // can't use the same request twice!

    } // end synchronized block

    if (authentication!=pcr.getAuthentication())
      throw new AccessError("Invalid password change request.");
    if (pcr.isExpired())
      throw new AccessError("Password change request has expired.");

    if (logger.isDebugEnabled())
      logger.debug("completePasswordChange for user \"" + pcr.getUserName() + "\"");

    // Request is valid; synthesize a new password
    PasswordGenerator pgen = new PasswordGenerator();
    PasswordHash phash = new PasswordHash(pgen.toString());
    Connection conn = null;
    try
    { // perform the database update
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("UPDATE users SET passhash = '");
      sql.append(phash.toString()).append("' WHERE uid = ").append(uid).append(';');
      stmt.executeUpdate(sql.toString());

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error resetting password for user: " + e.getMessage(),e);
      throw new DataException("unable to reset password: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

    // Create the message to be sent.
    String message = getStockMessage("password-change");
    if (message==null)
    { // what? the message does not exist?
      logger.error("\"password-change\" message does not exist in config file!");
      throw new DataException("INTERNAL: password change message not specified");

    } // end if

    // Replace the message variables.
    HashMap vars = new HashMap(2);
    vars.put("username",pcr.getUserName());
    vars.put("password",pgen.toString());
    message = StringUtil.replaceAllVariables(message,vars);

    // Find the message subject.
    String subject = getStockMessage("password-change-subject");
    if (subject==null)
      subject = "Venice Password Changed";

    // Create the emailer and send the message.
    Emailer emailer = createEmailer();
    emailer.setTo(pcr.getEmail());
    emailer.setSubject(subject);
    emailer.setText(message);
    emailer.send();
    if (logger.isDebugEnabled())
      logger.debug("...email sent");

  } // end completePasswordChange

  public UserContext createNewAccount(String remote_addr, String username, String password, String reminder)
      throws DataException, AccessError
  {
    checkInitialized();
    Connection conn = null;
    AuditRecord ar = null;
    String encode_username = SQLUtil.encodeString(username);
    Role new_role = global_security.getDefaultRole("Global.NewUser");
    // email confirmation # is between 1000000 and 9999999
    int confirm_num = getNewConfirmationNumber();
    int new_uid;
    java.util.Date created = null;

    if (logger.isDebugEnabled())
      logger.debug("createNewAccount(\"" + username + "\",<password>,<reminder>)...");
      
    try
    { // look to see if the user name is already present
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      stmt.executeUpdate("LOCK TABLES users WRITE, userprefs WRITE, propuser WRITE, sigmember WRITE, "
			 + "sideboxes WRITE, confhotlist WRITE;");
      try
      { // make sure the user name isn't there already
	ResultSet rs = stmt.executeQuery("SELECT uid FROM users WHERE username = '" + encode_username + "';");
	if (rs.next())
	{ // the user account already exists
	  logger.warn("username \"" + username + "\" already exists");
	  throw new AccessError("That user name already exists.  Please try again.");

	} // end if

	// perform the insert of the user's new data
	PasswordHash phash = new PasswordHash(password);
	StringBuffer sql = new StringBuffer("INSERT INTO users (username, passhash, email_confnum, "
					    + "base_lvl, created, lastaccess, passreminder) VALUES ('");
	sql.append(encode_username).append("', '").append(phash.toString()).append("', ");
	sql.append(confirm_num).append(", ").append(new_role.getLevel()).append(", '");
	created = new java.util.Date();
	sql.append(SQLUtil.encodeDate(created)).append("', '").append(SQLUtil.encodeDate(created));
	sql.append("', ").append(SQLUtil.encodeStringArg(reminder)).append(");");
	stmt.executeUpdate(sql.toString());

	// what is the new user ID?
	rs = stmt.executeQuery("SELECT LAST_INSERT_ID();");
	if (!(rs.next()))
	{ // the readback failed...
	  logger.error("readback of new user's UID failed");
	  throw new DataException("unable to read back new user ID");

	} // end if

	new_uid = rs.getInt(1);
	if (logger.isDebugEnabled())
	  logger.debug("...created user \"" + username + "\" with UID " + new_uid);

	// add a UserPrefs record for this user, too
	sql.setLength(0);
	sql.append("INSERT INTO userprefs (uid) VALUES (").append(new_uid).append(");");
	stmt.executeUpdate(sql.toString());

	if (logger.isDebugEnabled())
	  logger.debug("...created userprefs");

	// add a properties configuration for this user
	rs = stmt.executeQuery("SELECT propuser.ndx, propuser.data FROM propuser, users WHERE "
                               + "propuser.uid = users.uid AND users.is_anon = 1;");
	sql.setLength(0);
	while (rs.next())
	{ // set up to insert into the propuser table
	  if (sql.length()==0)
	    sql.append("INSERT INTO propuser (uid, ndx, data) VALUES ");
	  else
	    sql.append(", ");
	  sql.append("(").append(new_uid).append(", ").append(rs.getInt(1)).append(", '");
	  sql.append(SQLUtil.encodeString(rs.getString(2))).append("')");

	} // end while

	if (sql.length()>0)
	{ // execute the big update
	  sql.append(';');
	  stmt.executeUpdate(sql.toString());

	} // end if

	if (logger.isDebugEnabled())
	  logger.debug("...created user properties");

	// get the sidebox configuration for this user
	rs = stmt.executeQuery("SELECT sideboxes.boxid, sideboxes.sequence FROM sideboxes, "
			       + "users WHERE sideboxes.uid = users.uid AND users.is_anon = 1;");
	sql.setLength(0);
	while (rs.next())
	{ // set up to insert into the sideboxes table
	  if (sql.length()==0)
	    sql.append("INSERT INTO sideboxes (uid, boxid, sequence) VALUES ");
	  else
	    sql.append(", ");
	  sql.append("(").append(new_uid).append(", ").append(rs.getInt(1)).append(", ");
	  sql.append(rs.getInt(2)).append(')');

	} // end while

	if (sql.length()>0)
	{ // execute the big update
	  sql.append(';');
	  stmt.executeUpdate(sql.toString());

	} // end if

	if (logger.isDebugEnabled())
	  logger.debug("...loaded default sidebox config");

	// get the hotlist configuration for this user
	rs = stmt.executeQuery("SELECT confhotlist.sequence, confhotlist.sigid, confhotlist.confid FROM "
		             + "confhotlist, users WHERE confhotlist.uid = users.uid AND users.is_anon = 1;");
	sql.setLength(0);
	while (rs.next())
	{ // set up to insert into the confhotlist table
	  if (sql.length()==0)
	    sql.append("INSERT INTO confhotlist (uid, sequence, sigid, confid) VALUES ");
	  else
	    sql.append(", ");
	  sql.append('(').append(new_uid).append(", ").append(rs.getInt(1)).append(", ").append(rs.getInt(2));
	  sql.append(", ").append(rs.getInt(3)).append(')');

	} // end while

	if (sql.length()>0)
	{ // execute the big update
	  sql.append(';');
	  stmt.executeUpdate(sql.toString());

	} // end if

	if (logger.isDebugEnabled())
	  logger.debug("...loaded default hotlist config");

      } // end try
      finally
      { // make sure the tables get unlocked before we go
	Statement ulk_stmt = conn.createStatement();
	ulk_stmt.executeUpdate("UNLOCK TABLES;");

      } // end finally

      // the operation was a success - give back an audit record
      ar = new AuditRecord(AuditRecord.ACCOUNT_CREATE,new_uid,remote_addr);

    } // end try
    catch (SQLException e)
    { // database error - this is a DataException
      logger.error("DB error creating user: " + e.getMessage(),e);
      throw new DataException("unable to create user record: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      env.releaseConnection(conn);

    } // end finally

    // create a new context for the user (they're now effectively logged in)
    UserContextImpl rc = new UserContextImpl(env);
    rc.loadNewUser(remote_addr,new_uid,new_role.getLevel(),username,confirm_num,created,created);
    rc.autoJoinCommunities();  // EJB 4/14/2001
    if (logger.isDebugEnabled())
      logger.debug("...created new user context");
    return rc;

  } // end createNewAccount

  public boolean aliasExists(String alias, int exist_cid)
  {
    checkInitialized();
    Connection conn = null;

    try
    { // do a SELECT on the sigs table
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT sigid FROM sigs WHERE alias = '");
      sql.append(alias).append("'");
      if (exist_cid>0)
	sql.append(" AND sigid <> ").append(exist_cid);
      sql.append(';');
      ResultSet rs = stmt.executeQuery(sql.toString());
      return rs.next();

    } // end try
    catch (SQLException e)
    { // force to return null
      logger.warn("DB error looking up community: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

    return false;  // this is a "default" return

  } // end aliasExists

  public boolean isValidCategoryID(int catid)
  {
    checkInitialized();
    if (catid==-1)
      return true;  // valid by definition, it means "Top"
    Connection conn = null;

    try
    { // do a SELECT on the category table
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT catid FROM refcategory WHERE catid = ");
      sql.append(catid).append(';');
      ResultSet rs = stmt.executeQuery(sql.toString());
      return rs.next();

    } // end try
    catch (SQLException e)
    { // force to return null
      logger.warn("DB error looking up category: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

    return false;  // this is a "default" return

  } // end isValidCategoryID

  public List searchForUsers(int field, int mode, String term, int offset, int count) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("User search: field = " + field + ", mode = " + mode + ", term '" + term + "', offset = "
		   + offset + ", count = " + count);

    ArrayList rc = new ArrayList();  // return from this function
    Connection conn = null;          // pooled database connection

    try
    { // get a database connection
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT u.uid, u.username, u.description, c.given_name, "
					  + "c.family_name, c.locality, c.region, c.country FROM users u, "
					  + "contacts c WHERE u.contactid = c.contactid AND ");

      switch (field)
      { // determine which field to search on
	case SearchMode.FIELD_USER_NAME:
	  sql.append("u.username ");
	  break;

	case SearchMode.FIELD_USER_DESCRIPTION:
	  sql.append("u.description ");
	  break;

	case SearchMode.FIELD_USER_GIVEN_NAME:
	  sql.append("c.given_name ");
	  break;

	case SearchMode.FIELD_USER_FAMILY_NAME:
	  sql.append("c.family_name ");
	  break;

	default:
	  throw new DataException("invalid search field specified");

      } // end switch

      switch (mode)
      { // compose SQL in different ways depending on the search term
	case SearchMode.SEARCH_PREFIX:
	  sql.append("LIKE '").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_SUBSTRING:
	  sql.append("LIKE '%").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_REGEXP:
	  sql.append("REGEXP '").append(SQLUtil.encodeString(term)).append('\'');
	  break;

	default:
	  throw new DataException("invalid search mode specified");

      } // end switch

      sql.append(" AND u.is_anon = 0 ORDER BY u.username LIMIT ").append(offset).append(", ");
      sql.append(count+1).append(';');

      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // launch the search!
      ResultSet rs = stmt.executeQuery(sql.toString());

      while (rs.next())
      { // add all the found users to the list
	UserFound uf = new UserFoundImpl(rs.getInt(1),rs.getString(2),rs.getString(3),rs.getString(4),
					 rs.getString(5),rs.getString(6),rs.getString(7),rs.getString(8));
	rc.add(uf);

      } // end while

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading user entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve user information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      env.releaseConnection(conn);

    } // end finally

    return Collections.unmodifiableList(rc);

  } // end searchForUsers

  public int getSearchUserCount(int field, int mode, String term) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("User search: field = " + field + ", mode = " + mode + ", term '" + term + "'");

    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT COUNT(*) FROM users u, contacts c WHERE u.contactid = "
					  + "c.contactid AND ");

      switch (field)
      { // determine which field to search on
	case SearchMode.FIELD_USER_NAME:
	  sql.append("u.username ");
	  break;

	case SearchMode.FIELD_USER_DESCRIPTION:
	  sql.append("u.description ");
	  break;

	case SearchMode.FIELD_USER_GIVEN_NAME:
	  sql.append("c.given_name ");
	  break;

	case SearchMode.FIELD_USER_FAMILY_NAME:
	  sql.append("c.family_name ");
	  break;

	default:
	  throw new DataException("invalid search field specified");

      } // end switch

      switch (mode)
      { // compose SQL in different ways depending on the search term
	case SearchMode.SEARCH_PREFIX:
	  sql.append("LIKE '").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_SUBSTRING:
	  sql.append("LIKE '%").append(SQLUtil.encodeStringWildcards(term)).append("%'");
	  break;

	case SearchMode.SEARCH_REGEXP:
	  sql.append("REGEXP '").append(SQLUtil.encodeString(term)).append('\'');
	  break;

	default:
	  throw new DataException("invalid search mode specified");

      } // end switch

      sql.append(" AND u.is_anon = 0;");


      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // launch the search!
      ResultSet rs = stmt.executeQuery(sql.toString());

      if (!(rs.next()))
	throw new InternalStateError("getSearchUserCount search failure (MUST have 1 row!)");

      return rs.getInt(1);

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading user entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve user information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      env.releaseConnection(conn);

    } // end finally

  } // end getSearchUserCount

  public int getStdNumSearchResults()
  {
    return gp_ints[IP_MAXSEARCHRETURN];

  } // end getStdNumSearchResults

  public boolean isEmailAddressBanned(String email)
  {
    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = env.getConnection();
      Statement stmt = conn.createStatement();

      // create and execute the right SQL statement
      StringBuffer sql = new StringBuffer("SELECT by_uid FROM emailban WHERE address = '");
      sql.append(SQLUtil.encodeString(email)).append("';");
      ResultSet rs = stmt.executeQuery(sql.toString());
      return rs.next();

    } // end try
    catch (SQLException e)
    { // if the database is horked, return false
      logger.warn("DB error looking email address: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      env.releaseConnection(conn);

    } // end finally

    return false;

  } // end isEmailAddressBanned

  public boolean confAliasExists(String alias)
  {
    checkInitialized();
    Connection conn = null;

    try
    { // do a SELECT on the confalias table
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      StringBuffer sql = new StringBuffer("SELECT confid FROM confalias WHERE alias = '");
      sql.append(alias).append("';");
      ResultSet rs = stmt.executeQuery(sql.toString());
      return rs.next();

    } // end try
    catch (SQLException e)
    { // force to return false
      logger.warn("DB error looking up conference: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

    return false;  // this is a "default" return

  } // end confAliasExists

  public HTMLChecker getEscapingChecker()
  {
    return html_configs[HTMLC_ESCAPE_BODY_PSEUD].createHTMLChecker();

  } // end getEscapingChecker

  public int getNumPostsPerPage()
  {
    return gp_ints[IP_POSTSPERPAGE];

  } // end getNumPostsPerPage

  public int getNumOldPostsBeforeNew()
  {
    return gp_ints[IP_POSTSATTOP];

  } // end getNumOldPostsBeforeNew

  public int getMaxNumConferenceMembersDisplay()
  {
    return gp_ints[IP_MAXCONFMEMBERDISPLAY];

  } // end getMaxNumConferenceMembersDisplay

  public int getMaxNumCommunityMembersDisplay()
  {
    return gp_ints[IP_MAXCOMMUNITYMEMBERDISPLAY];

  } // end getMaxNumCommunityMembersDisplay

  public List getMasterSideBoxList()
  {
    return new MasterSideBoxList(sideboxes);

  } // end getMasterSideBoxList

  public List getPublishedMessages(boolean all) throws DataException
  {
    ArrayList rc = new ArrayList();

    synchronized (this)
    { // Make sure the cache is in condition.
      while (cache_fp_posts_busy)
      { // the list is busy, don't fsck with it until we're done
	try
	{ // wait to see if they can be un-busy
	  wait();

	} // end try
	catch (InterruptedException e)
	{ // do nothing
	} // end catch

      } // end while

      // If the cache list contains too few items, backfill it from the database.
      if (cache_fp_posts.size()<gp_ints[IP_NUMFRONTPAGEPOSTS])
	PublishedMessageImpl.backfillCache(cache_fp_posts,gp_ints[IP_NUMFRONTPAGEPOSTS],env);

      // Copy the contents to the return vector.
      Iterator it = cache_fp_posts.iterator();
      while (it.hasNext())
      { // cast each element and add the casted object to the output
	TopicMessageContext ctxt = (TopicMessageContext)(it.next());
	rc.add(ctxt);

      } // end while

    } // end synchronized block

    if (all)  // add the extra postings to the list
      PublishedMessageImpl.backfillReturn(rc,env);

    return Collections.unmodifiableList(rc);

  } // end getPublishedMessages

  public int getNumAuditRecordsPerPage()
  {
    return gp_ints[IP_NUMAUDITRECSPERPAGE];

  } // end getNumAuditRecordsPerPage

  public Advertisement getAdByID(int id)
  {
    return AdvertisementImpl.getAdByID(env,id);

  } // end getAdByID

  public Advertisement selectAd()
  {
    // just get a random ad for now
    return AdvertisementImpl.getRandomAd(env);

  } // end selectAd

  public BinaryData loadImage(int id) throws DataException
  {
    return ImageStore.loadImageByID(env,id);

  } // end loadImage

  public Dimension getUserPhotoSize()
  {
    return DEFAULT_DIM_USERPHOTO;

  } // end getUserPhotoSize

  public Dimension getCommunityLogoSize()
  {
    return DEFAULT_DIM_COMMUNITYLOGO;

  } // end getCommunityLogoSize

  public SecurityInfo getSecurityInfo()
  {
    return new SecurityInfoWrapper(global_security);

  } // end getSecurityInfo

  /*--------------------------------------------------------------------------------
   * Implementations from interface EngineBackend
   *--------------------------------------------------------------------------------
   */

  public VeniceEngine getSelf()
  {
    return this;

  } // end getSelf

  public Emailer createEmailer()
  {
    checkInitialized();
    return new SimpleEmailer(email_props,mailsession);

  } // end createEmailer

  public String getStockMessage(String key)
  {
    checkInitialized();
    return (String)(stock_messages.get(key));

  } // end getStockMessage

  public int getNewConfirmationNumber()
  {
    checkInitialized();
    return rng.nextInt(9000000) + 1000000;

  } // end getNewConfirmationNumber

  public CommunityData getCommunityDataObject(int cid) throws DataException
  {
    checkInitialized();
    if (logger.isDebugEnabled())
      logger.debug("getCommunityDataObject(" + cid + ")...");

    try
    { // delegate to the comm_objcache object
      return (CommunityData)(comm_objcache.getOrCreate(new Integer(cid)));

    } // end try
    catch (ObjectFactoryException e)
    { // this may be a DataException, or it may not
      Throwable e2 = e.getException();
      if (e2 instanceof DataException)
	throw (DataException)e2;
      else
	throw new InternalStateError("unknown creation exception thrown in getCommunityDataObject: "
				     + e2.getClass().getName(),e2);

    } // end catch

  } // end getCommunityDataObject

  public void detachCommunityDataObject(int cid)
  {
    checkInitialized();
    if (logger.isDebugEnabled())
      logger.debug("detachCommunityDataObject(" + cid + ")...");

    comm_objcache.detach(new Integer(cid));

  } // end detachCommunityDataObject

  public BitSet getLockedFeaturesMask()
  {
    checkInitialized();
    BitSet rc = new BitSet();
    for (int i=0; i<features.length; i++)
      if (features[i].isLocked())
	rc.set(i);

    return rc;

  } // end getLockedFeaturesMask

  public List getCommunityFeatureSet(BitSet enabled_features, int level, boolean read_privs)
  {
    checkInitialized();
    ArrayList rc = new ArrayList();
    for (int i=0; i<features.length; i++)
      if (enabled_features.get(i) && features[i].featureAllowed(level,read_privs))
      { // this feature must be included in the eventual output set
	CommunityFeature elt = features[i];
	boolean insert_me = true;
	for (int j=0; insert_me && (j<rc.size()); j++)
	  if (elt.getSequence()<((CommunityFeature)(rc.get(j))).getSequence())
	  { // put the item in the list
	    rc.add(j,elt);
	    insert_me = false;

	  } // end if and for

	if (insert_me)  // insert at end by default
	  rc.add(elt);

      } // end if and for

    if (logger.isDebugEnabled())
      logger.debug("getCommunityFeatureSet() loaded " + rc.size() + " elements");

    return Collections.unmodifiableList(rc);  // wrap the vector for return

  } // end getCommunityFeatureSet

  public String getAppletForFeature(int code)
  {
    checkInitialized();
    if ((code<0) || (code>=features.length))
    { // the index is out of bounds!
      logger.error("getAppletForFeature(" + code + ") => out of range");
      throw new IndexOutOfBoundsException("invalid feature code");

    } // end if

    String rc = features[code].getApplet();
    if (logger.isDebugEnabled())
      logger.debug("getAppletForFeature(" + code + ") => \"" + rc + "\"");
    return rc;

  } // end getAppletForFeature

  public boolean isValidInitialFeatureIndex(short ndx)
  {
    if ((ndx<0) || (ndx>=features.length))
      return false;
    return features[ndx].isVisible();

  } // end isValidInitialFeatureIndex

  public BitSet getDefaultFeaturesMask()
  {
    checkInitialized();
    BitSet rc = new BitSet();
    for (int i=0; i<features.length; i++)
      if (features[i].isDefault())
	rc.set(i);

    return rc;

  } // end getDefaultFeaturesMask

  public void registerNewCommunity(CommunityData comm)
  {
    checkInitialized();
    if (logger.isDebugEnabled())
      logger.debug("registerNewCommunity(" + comm.getID() + ")...");

    comm_objcache.register(new Integer(comm.getID()),comm);

  } // end registerNewCommunity

  public int getFeatureIndexBySymbol(String symbol)
  {
    VeniceFeatureDef fd = (VeniceFeatureDef)(feature_syms.get(symbol));
    if (fd!=null)
      return fd.getFeatureCode();
    else
      return -1;

  } // end getFeatureIndexBySymbol

  public boolean canAccessFeature(String symbol, int level, boolean has_read)
  {
    VeniceFeatureDef fd = (VeniceFeatureDef)(feature_syms.get(symbol));
    if (fd!=null)
      return fd.featureAllowed(level,has_read);
    else
      return false;

  } // end canAccessFeature

  public ConferenceData getConferenceDataObject(int confid) throws DataException
  {
    checkInitialized();
    if (logger.isDebugEnabled())
      logger.debug("getConferenceDataObject(" + confid + ")...");

    try
    { // delegate to the conf_objcache objects
      return (ConferenceData)(conf_objcache.getOrCreate(new Integer(confid)));

    } // end try
    catch (ObjectFactoryException e)
    { // this may be a DataException, or it may not
      Throwable e2 = e.getException();
      if (e2 instanceof DataException)
	throw (DataException)e2;
      else
	throw new InternalStateError("unknown creation exception thrown in getConferenceDataObject: "
				     + e2.getClass().getName(),e2);

    } // end catch

  } // end getConferenceDataObject

  public void detachConferenceDataObject(int confid)
  {
    checkInitialized();
    if (logger.isDebugEnabled())
      logger.debug("detachConferenceDataObject(" + confid + ")...");

    conf_objcache.detach(new Integer(confid));

  } // end detachConferenceDataObject

  public void saveAuditRecord(AuditRecord ar)
  {
    checkInitialized();
    Connection conn = null;

    if (ar==null)
      return;  // don't store a null record

    try
    { // get a connection and use it to store the audit record
      conn = env.getConnection();
      ar.store(conn);

    } // end try
    catch (SQLException e)
    { // just log an error if we screwed up
      logger.error("DB error saving audit record: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end saveAuditRecord

  public void registerNewConference(ConferenceData conf)
  {
    checkInitialized();
    if (logger.isDebugEnabled())
      logger.debug("registerNewConference(" + conf.getID() + ")...");

    conf_objcache.register(new Integer(conf.getID()),conf);

  } // end registerNewConference

  public HTMLChecker createCheckerObject(int type)
  {
    if ((type<0) || (type>=html_configs.length))
      throw new IllegalArgumentException("invalid HTML checker type index");
    return html_configs[type].createHTMLChecker();

  } // end createCheckerObject

  public int getParamInt(int selector)
  {
    return gp_ints[selector];

  } // end getParamInt

  public boolean getParamBoolean(int selector)
  {
    return global_flags.get(selector);

  } // end getParamBoolean

  public Role getParamRole(int selector)
  {
    if (selector==ROLEP_CREATECOMMUNITY)
      return global_security.getRoleForLevel(gp_ints[IP_CREATECOMMUNITYLVL]);
    return null;

  } // end getParamRole

  public void forceParamReload() throws DataException
  {
    Connection conn = null;  // data pooled connection

    try
    { // get a connection and use it to reload
      conn = env.getConnection();
      Statement stmt = conn.createStatement();
      loadDefaults(stmt);

    } // end try
    catch (SQLException e)
    { // just log an error if we screwed up
      logger.error("DB error loading parameters: " + e.getMessage(),e);
      throw new DataException("Database error reloading parameters: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure the connection is released before we go
      env.releaseConnection(conn);

    } // end finally

  } // end forceParamReload

  public SideBoxDescriptor getMasterSideBoxDescriptor(int id)
  {
    return (SideBoxDescriptor)(sidebox_ids.get(new Integer(id)));

  } // end getMasterSideBoxDescriptor

  public synchronized void startPublish()
  {
    cache_fp_posts_busy = true;

  } // end startPublish

  public synchronized void publishNew(TopicMessageContext pubmsg)
  {
    if (pubmsg!=null)
    { // add the new message
      cache_fp_posts.addFirst(pubmsg);
      while (cache_fp_posts.size()>gp_ints[IP_NUMFRONTPAGEPOSTS])
	cache_fp_posts.removeLast();

    } // end pubmsg

    cache_fp_posts_busy = false;
    notifyAll();

  } // end publishNew

  public synchronized void unpublish(long postid)
  {
    Iterator it = cache_fp_posts.iterator();
    while (it.hasNext())
    { // get each published message in the cache in turn, looking for the specified post ID
      TopicMessageContext tmc = (TopicMessageContext)(it.next());
      if (tmc.getPostID()==postid)
      { // drop the specified post ID like a hot rock
	it.remove();
	break;

      } // end if

    } // end while

  } // end unpublish

  public String generateRandomAuthString()
  {
    StringBuffer buf = new StringBuffer(AUTH_STRING_LEN);
    for (int i=0; i<AUTH_STRING_LEN; i++)
      buf.append(AUTH_ALPHABET.charAt(rng.nextInt(AUTH_ALPHABET.length())));
    return buf.toString();

  } // end generateRandomAuthString

  public boolean isValidRandomAuthString(String s)
  {
    if (s.length()!=AUTH_STRING_LEN)
      return false;
    for (int i=0; i<AUTH_STRING_LEN; i++)
    { // verify each authentication character in turn
      if (AUTH_ALPHABET.indexOf(s.charAt(i))<0)
	return false;

    } // end for

    return true;  // all tests passed - ship it!

  } // end isValidRandomAuthString

  public boolean isNoCompressMimeType(String type)
  {
    if (no_compress_types.isEmpty())
      return false;  // can't possibly be no-compress

    // strip the MIME type name down to basics
    int psem = type.indexOf(';');
    String real_type;
    if (psem>=0)
      real_type = type.substring(0,psem).trim().toLowerCase();
    else
      real_type = type.trim().toLowerCase();

    return no_compress_types.contains(real_type);

  } // end isNoCompressMimeType

  public GlobalProperties getProperties()
  {
    return createProperties();

  } // end getProperties

  public void setProperties(GlobalProperties props) throws DataException
  {
    GlobalProperties orig = createProperties();
    BitSet delta = storeProperties(props);
    if (delta.length()==0)
      return;

    boolean succeeded = false;
    try
    { // write the changes out to the database
      updateDefaults(delta);
      succeeded = true;

    } // end try
    finally
    { // restore old properties on failure
      if (!succeeded)
	storeProperties(orig);

    } // end finally

  } // end setProperties

  public SecurityMonitor env_getSecurityMonitor(int selector)
  {
    if (selector==SM_GLOBAL)
      return global_security;
    if (selector==SM_COMMUNITY)
      return community_security;
    return null;

  } // end env_getSecurityMonitor

  /*--------------------------------------------------------------------------------
   * Static initializer
   *--------------------------------------------------------------------------------
   */

  static
  { // Initialize the "integer parameter properties" BitSet.
    BitSet tmp = new BitSet();
    tmp.set(_PPROP_IP_BASE + IP_POSTSPERPAGE);
    tmp.set(_PPROP_IP_BASE + IP_POSTSATTOP);
    tmp.set(_PPROP_IP_BASE + IP_MAXSEARCHRETURN);
    tmp.set(_PPROP_IP_BASE + IP_MAXCOMMUNITYMEMBERDISPLAY);
    tmp.set(_PPROP_IP_BASE + IP_MAXCONFMEMBERDISPLAY);
    tmp.set(_PPROP_IP_BASE + IP_NUMFRONTPAGEPOSTS);
    tmp.set(_PPROP_IP_BASE + IP_NUMAUDITRECSPERPAGE);
    tmp.set(_PPROP_IP_BASE + IP_CREATECOMMUNITYLVL);
    IP_PROPS = tmp;

    // Initialize the "all properties" BitSet.
    tmp = new BitSet();
    tmp.set(PROP_FLAGS);
    tmp.or(IP_PROPS);
    ALL_PROPS = tmp;

  } // end static initializer

} // end class VeniceEngineImpl
