/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.security;

import java.io.IOException;
import java.util.*;
import org.apache.log4j.*;
import com.silverwrist.venice.core.AccessError;

public class PrimordialSecurityMonitor implements SecurityMonitor
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static PrimordialSecurityMonitor self = null;

  private static Category logger = Category.getInstance(PrimordialSecurityMonitor.class);

  public static final String SYM_NOT_IN_LIST = "NotInList";
  public static final String SYM_NO_ACCESS = "NoAccess";
  public static final String SYM_UNRESTRICTED = "UnrestrictedUser";

  private static final String NAME_NOT_IN_LIST = "(not in list)";
  private static final String NAME_NO_ACCESS = "No Access";
  private static final String NAME_UNRESTRICTED = "'Unrestricted' User";

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private Role not_in_list;              // the "not in list" role
  private Role no_access;                // the "no access" role
  private Role unrestricted_user;        // the "unrestricted user" role

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  private PrimordialSecurityMonitor()
  {
    String name_not_in_list = null, name_no_access = null, name_unrestricted_user = null;

    try
    { // load the three initial role names
      Properties init_props = new Properties();
      init_props.load(getClass().getResourceAsStream("sm.properties"));
      name_not_in_list = init_props.getProperty("role.NotInList.name",NAME_NOT_IN_LIST);
      name_no_access = init_props.getProperty("role.NoAccess.name",NAME_NO_ACCESS);
      name_unrestricted_user = init_props.getProperty("role.UnrestrictedUser.name",NAME_UNRESTRICTED);

    } // end try
    catch (IOException e)
    { // on error, revert to the internal names
      logger.warn("Error loading default names",e);
      name_not_in_list = NAME_NOT_IN_LIST;
      name_no_access = NAME_NO_ACCESS;
      name_unrestricted_user = NAME_UNRESTRICTED;

    } // end catch

    // Create the actual roles.
    this.not_in_list = Role.create(ScopeInfo.L_NOT_THERE,name_not_in_list,SYM_NOT_IN_LIST);
    this.no_access = Role.create(ScopeInfo.L_NO_ACCESS,name_no_access,SYM_NO_ACCESS);
    this.unrestricted_user = Role.create(ScopeInfo.L_UNRESTRICTED,name_unrestricted_user,SYM_UNRESTRICTED);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Implementations from interface SecurityMonitor
   *--------------------------------------------------------------------------------
   */

  public boolean testPermission(String symbol, int level, String errormsg) throws AccessError
  {
    if (symbol==null)
      throw new NullPointerException("testPermission() got null symbol");
    logger.error("testPermission: symbol \"" + symbol + "\" is not a defined permission test");
    throw new AccessError("Undefined internal permission test \"" + symbol + "\"");

  } // end testPermission

  public boolean testPermission(String symbol, int level)
  {
    if (symbol==null)
      throw new NullPointerException("testPermission() got null symbol");
    logger.error("testPermission: symbol \"" + symbol + "\" is not a defined permission test");
    return false;

  } // end testPermission

  public boolean permissionDefined(String symbol, boolean no_follow)
  {
    if (symbol==null)
      throw new NullPointerException("permissionDefined() got null symbol");
    return false;  // primordial monitor defines no permissions

  } // end permissionDefined

  public List getRoleList(String symbol)
  {
    if (symbol==null)
      throw new NullPointerException("getRoleList() got null symbol");
    logger.error("getRoleList: symbol \"" + symbol + "\" is not a defined role list");
    return null;

  } // end getRoleList

  public Role getRole(String symbol)
  {
    if (symbol==null)
      throw new NullPointerException("getRole() got null symbol");
    if (symbol.equals(SYM_NOT_IN_LIST))
      return not_in_list;
    if (symbol.equals(SYM_NO_ACCESS))
      return no_access;
    if (symbol.equals(SYM_UNRESTRICTED))
      return unrestricted_user;
    logger.error("getRole: symbol \"" + symbol + "\" is not a defined role");
    return null;

  } // end getRole

  public Role getRoleForLevel(int level)
  {
    if (level==ScopeInfo.L_NOT_THERE)
      return not_in_list;
    if (level==ScopeInfo.L_NO_ACCESS)
      return no_access;
    if (level==ScopeInfo.L_UNRESTRICTED)
      return unrestricted_user;
    logger.error("getRoleForLevel: no role for level " + level);
    return null;

  } // end getRoleForLevel

  public Role getDefaultRole(String symbol)
  {
    logger.error("getDefaultRole: symbol \"" + symbol + "\" has no default role");
    return null;

  } // end getDefaultRole

  public ScopeInfo getScopeInfo()
  {
    return null;  // this security monitor HAS no scope

  } // end getScopeInfo

  public String getID()
  {
    return null;  // this security monitor HAS no ID

  } // end getID

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static synchronized SecurityMonitor get()
  {
    if (self==null)
      self = new PrimordialSecurityMonitor();
    return self;

  } // end get

} // end class PrimordialSecurityMonitor
