/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.security;

import java.util.*;

public final class Role implements Comparable, Cloneable, SecLevels
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Role not_in_list;
  private static Role no_access;
  private static Role unrestricted_user;
  private static Role global_admin;
  private static Role comm_host;
  private static ArrayList global_low;
  private static ArrayList global_high;
  private static ArrayList comm_low;
  private static ArrayList comm_high;
  private static ArrayList conf_low;
  private static ArrayList conf_high;

  private static List confreadlist_rc = null;
  private static List confpostlist_rc = null;
  private static List confhidelist_rc = null;
  private static List confdeletelist_rc = null;
  private static List conf_member_levels = null;

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private int level;
  private String name;
  private String symbol;

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  protected Role(int level, String name)
  {
    this.level = level;
    this.name = name;
    this.symbol = null;

  } // end constructor

  protected Role(int level, String name, String symbol)
  {
    this.level = level;
    this.name = name;
    this.symbol = symbol;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class Object
   *--------------------------------------------------------------------------------
   */

  public boolean equals(Object obj)
  {
    if (obj==null)
      return false;
    else if (obj==this)
      return true;
    if (obj instanceof Role)
    { // compare levels
      Role other = (Role)obj;
      return (level==other.level);

    } // end if

    return obj.toString().equals(toString());

  } // end equals

  public int hashCode()
  {
    return level;

  } // end hashCode

  public String toString()
  {
    StringBuffer buf = new StringBuffer(name);
    buf.append('[').append(level).append(']');
    if (symbol!=null)
      buf.append('{').append(symbol).append('}');
    return buf.toString();

  } // end toString

  /*--------------------------------------------------------------------------------
   * Implementations from interface Comparable
   *--------------------------------------------------------------------------------
   */

  public int compareTo(Object obj)
  {
    if (obj==null)
      throw new NullPointerException("comparing a NULL object");
    if (!(obj instanceof Role))
      throw new ClassCastException("comparing non-Role to Role");
    Role other = (Role)obj;
    return level - other.level;

  } // end compareTo

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public final int getLevel()
  {
    return level;

  } // end getLevel

  public final String getName()
  {
    return name;

  } // end getName

  public final String getSymbol()
  {
    return symbol;

  } // end getSymbol

  public final boolean isSatisfiedBy(int l)
  {
    return (l>=level);

  } // end isSatisfiedBy

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static final Role create(int level, String name, String symbol)
  {
    return new Role(level,name,symbol);

  } // end create

  /*--------------------------------------------------------------------------------
   * External static operations which generate lists of roles
   *--------------------------------------------------------------------------------
   */

  public static List getConferenceReadList()
  {
    if (confreadlist_rc==null)
    { // precalculate the conference read list
      ArrayList rc = new ArrayList();
      rc.addAll(global_low);
      rc.addAll(comm_low);
      rc.addAll(conf_low);
      rc.add(unrestricted_user);
      rc.trimToSize();
      confreadlist_rc = Collections.unmodifiableList(rc);

    } // end if

    return confreadlist_rc;

  } // end getConferenceReadList

  public static List getConferencePostList()
  {
    if (confpostlist_rc==null)
    { // precalculate the post list
      ArrayList rc = new ArrayList();
      rc.addAll(global_low);
      rc.addAll(comm_low);
      rc.addAll(conf_low);
      rc.add(unrestricted_user);
      rc.addAll(conf_high);
      rc.trimToSize();
      confpostlist_rc = Collections.unmodifiableList(rc);

    } // end if

    return confpostlist_rc;

  } // return getConferencePostList

  public static List getConferenceCreateList()
  {
    return getConferencePostList();

  } // end getConferenceChangeList

  public static List getConferenceHideList()
  {
    if (confhidelist_rc==null)
    { // precalculate the hide list
      ArrayList rc = new ArrayList();
      rc.addAll(conf_high);
      rc.addAll(comm_high);
      rc.add(global_high.get(0));
      rc.trimToSize();
      confhidelist_rc = Collections.unmodifiableList(rc);

    } // end if

    return confhidelist_rc;

  } // end getConferenceHideList

  public static List getConferenceNukeList()
  {
    return getConferenceHideList();

  } // end getConferenceNukeList

  public static List getConferenceChangeList()
  {
    return getConferenceHideList();

  } // end getConferenceChangeList

  public static List getConferenceDeleteList()
  {
    if (confdeletelist_rc==null)
    { // precalculate the delete list
      ArrayList rc = new ArrayList();
      rc.addAll(comm_high);
      rc.addAll(global_high);
      rc.add(no_access);
      rc.trimToSize();
      confdeletelist_rc = Collections.unmodifiableList(rc);

    } // end if

    return confdeletelist_rc;

  } // end getConferenceDeleteList

  public static List getConferenceMemberLevelChoices()
  {
    if (conf_member_levels==null)
    { // precalculate the list
      ArrayList rc = new ArrayList();
      rc.add(not_in_list);
      rc.addAll(global_low);
      rc.addAll(comm_low);
      rc.addAll(conf_low);
      rc.add(unrestricted_user);
      rc.add(conf_high.get(conf_high.size()-1));
      rc.trimToSize();
      conf_member_levels = Collections.unmodifiableList(rc);

    } // end if
    
    return conf_member_levels;

  } // end getConferenceMemberLevelChoices

  /*--------------------------------------------------------------------------------
   * Static initializer
   *--------------------------------------------------------------------------------
   */

  static
  { // begin initializing the "all roles" map
    not_in_list = new Role(0,"(not in list)");
    no_access = new Role(NO_ACCESS,"No Access");
    unrestricted_user = new Role(UNRESTRICTED_USER,"'Unrestricted' User");

    Role tmp;

    // initialize the "global lowband" vector
    global_low = new ArrayList(3);
    tmp = new Role(GLOBAL_ANONYMOUS,"Anonymous User");
    global_low.add(tmp);
    tmp = new Role(GLOBAL_UNVERIFIED,"Unauthenticated User");
    global_low.add(tmp);
    tmp = new Role(GLOBAL_NORMAL,"Normal User");
    global_low.add(tmp);
    global_low.trimToSize();

    // initialize the "global highband" vector
    global_high = new ArrayList(3);
    tmp = new Role(GLOBAL_ANYADMIN,"Any System Administrator");
    global_high.add(tmp);
    tmp = new Role(GLOBAL_PFY,"System Assistant Administrator");
    global_high.add(tmp);
    global_admin = new Role(GLOBAL_BOFH,"Global System Administrator");
    global_high.add(global_admin);
    global_high.trimToSize();

    // initialize the "community lowband" vector
    comm_low = new ArrayList(1);
    tmp = new Role(COMM_MEMBER,"Community Member");
    comm_low.add(tmp);
    comm_low.trimToSize();

    // initialize the "communtiy highband" vector
    comm_high = new ArrayList(3);
    tmp = new Role(COMM_ANYADMIN,"Any Community Administrator");
    comm_high.add(tmp);
    tmp = new Role(COMM_COHOST,"Community Co-Host");
    comm_high.add(tmp);
    comm_host = new Role(COMM_HOST,"Community Host");
    comm_high.add(comm_host);
    comm_high.trimToSize();

    // initialize the "conference lowband" vector
    conf_low = new ArrayList(1);
    tmp = new Role(CONFERENCE_MEMBER,"Conference Member");
    conf_low.add(tmp);
    conf_low.trimToSize();

    // initialize the "conference highband" vector
    conf_high = new ArrayList(2);
    tmp = new Role(CONFERENCE_ANYADMIN,"Any Conference Administrator");
    conf_high.add(tmp);
    tmp = new Role(CONFERENCE_HOST,"Conference Host");
    conf_high.add(tmp);
    conf_high.trimToSize();

  } // end static initializer

} // end class Role
