/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.security;

public final class ScopeInfo implements Cloneable, Comparable
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static final int[] LB_LOW =   // Scope values, lowband, low end of range
      { 0, 2000, 4000, 6000, 8000, 10000, 12000, 14000, 16000, 18000, 20000, 22000, 24000, 26000, 28000,
	30000 };
  private static final int[] LB_HIGH =  // Scope values, lowband, high end of range
      { 1999, 3999, 5999, 7999, 9999, 11999, 13999, 15999, 17999, 19999, 21999, 23999, 25999, 27999, 29999,
	31999 };
  private static final int[] HB_LOW =   // Scope values, highband, low end of range
      { 63000, 61000, 59000, 57000, 55000, 53000, 51000, 49000, 47000, 45000, 43000, 41000, 39000, 37000,
	35000, 33000 };
  private static final int[] HB_HIGH =  // Scope values, highband, high end of range
      { 64999, 62999, 60999, 58999, 56999, 54999, 52999, 50999, 48999, 46999, 44999, 42999, 40999, 38999,
	36999, 34999 };

  public static final int L_NOT_THERE = -1;         // global "not there" constant
  public static final int L_UNRESTRICTED = 32500;   // global "unrestricted user" constant
  public static final int L_NO_ACCESS = 65500;      // global "no access" constant

  public static final int BAND_WIDTH = 1999;        // offset between "high and low" values within a band

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  int scope;                 // the scope value

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public ScopeInfo(int scope)
  {
    if ((scope<0) || (scope>=LB_LOW.length))
      throw new IndexOutOfBoundsException("invalid scope value");
    this.scope = scope;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class Object
   *--------------------------------------------------------------------------------
   */

  public boolean equals(Object o)
  {
    if ((o==null) || !(o instanceof ScopeInfo))
      return false;
    ScopeInfo other = (ScopeInfo)o;
    return (scope==other.scope);

  } // end equals

  public int hashCode()
  {
    return scope;

  } // end hashCode

  public String toString()
  {
    StringBuffer buf = new StringBuffer("{ScopeInfo(");
    buf.append(scope).append("): ranges [").append(LB_LOW[scope]).append('-').append(LB_HIGH[scope]);
    buf.append("], [").append(HB_LOW[scope]).append('-').append(HB_HIGH[scope]).append("]}");
    return buf.toString();

  } // end toString

  /*--------------------------------------------------------------------------------
   * Implementations from interface Comparable
   *--------------------------------------------------------------------------------
   */

  public int compareTo(Object o)
  {
    if (o==null)
      throw new NullPointerException("can't compare to a null object");
    ScopeInfo other = (ScopeInfo)o;  // may throw ClassCastException - that's OK
    return scope - other.scope;

  } // end compareTo

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public final int getScope()
  {
    return scope;

  } // end getScope

  public final int getLowBandLow()
  {
    return LB_LOW[scope];

  } // end getLowBandLow

  public final int getLowBandHigh()
  {
    return LB_HIGH[scope];

  } // end getLowBandHigh

  public final int getHighBandLow()
  {
    return HB_LOW[scope];

  } // end getHighBandLow

  public final int getHighBandHigh()
  {
    return HB_HIGH[scope];

  } // end getHighBandHigh

  public final int getLevel(boolean highband, int offset)
  {
    int rc;
    if (highband)
    { // it's in the highband
      if (offset<0)
      { // negative offset from high end of highband
	rc = HB_HIGH[scope] + offset;
	if (rc<HB_LOW[scope])
	  throw new IllegalArgumentException("value out of scope");

      } // end if
      else
      { // positive offset from low end of highband
	rc = HB_LOW[scope] + offset;
	if (rc>HB_HIGH[scope])
	  throw new IllegalArgumentException("value out of scope");

      } // end else

    } // end if (highband)
    else
    { // it's in the lowband
      if (offset<0)
      { // negative offset from high end of lowband
	rc = LB_HIGH[scope] + offset;
	if (rc<LB_LOW[scope])
	  throw new IllegalArgumentException("value out of scope");

      } // end if
      else
      { // positive offset from low end of lowband
	rc = LB_LOW[scope] + offset;
	if (rc>LB_HIGH[scope])
	  throw new IllegalArgumentException("value out of scope");

      } // end else

    } // end else (lowband)

    return rc;

  } // end getLevel

  public final boolean isInScope(int value)
  {
    if ((value>=LB_LOW[scope]) && (value<=LB_HIGH[scope]))
      return true;
    if ((value>=HB_LOW[scope]) && (value<=HB_HIGH[scope]))
      return true;
    return false;

  } // end isInScope

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static final boolean isValidScope(int s)
  {
    return ((s>=0) && (s<LB_LOW.length));

  } // end isValidScope

  public static final int getScopeOf(int value)
  {
    if ((value<LB_LOW[0]) || (value>HB_HIGH[0]))
      return -1;  // quick test to eliminate most of the range

    for (int i=0; i<LB_LOW.length; i++)
    { // look in each scope in turn
      if ((value<=LB_HIGH[i]) || (value>=HB_LOW[i]))
	return i;

    } // end for

    return -1;  // not in a scope

  } // end getScopeOf

} // end class ScopeInfo
