/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.silverwrist.venice.ValidationException;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class Find extends VeniceServlet
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  protected static final String DISPLAY_PARAM_ATTRIBUTE = "servlet.Find.display";

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private static int getDisplayParam(HttpServletRequest request, int nchoices)
  {
    HttpSession session = request.getSession(true);

    try
    { // attempt to get the display parameter
      String dp_str = request.getParameter("disp");
      if (dp_str!=null)
      { // attempt to convert to an integer and range-check it
	int dp = Integer.parseInt(dp_str);
	if ((dp>=0) && (dp<nchoices))
	{ // this is our new current display parameter
	  session.setAttribute(DISPLAY_PARAM_ATTRIBUTE,new Integer(dp));
	  return dp;

	} // end if

      } // end if (parameter found)

    } // end try
    catch (NumberFormatException nfe)
    { // do nothing - fall through and get the stored default
    } // end catch

    // retrieve the stored default from the HTTP session
    Integer dp_default = (Integer)(session.getAttribute(DISPLAY_PARAM_ATTRIBUTE));
    if (dp_default==null)
    { // create an initial default and save it
      dp_default = new Integer(0);
      session.setAttribute(DISPLAY_PARAM_ATTRIBUTE,dp_default);

    } // end if

    return dp_default.intValue();

  } // end getDisplayParam

  private int getCategoryParam(VeniceEngine engine, ServletRequest request)
  {
    String cat_str = request.getParameter("cat");
    if (cat_str==null)
      return -1;

    try
    { // get the category ID and check it for validity
      int cat = Integer.parseInt(cat_str);
      if (engine.isValidCategoryID(cat))
	return cat;

    } // end try
    catch (NumberFormatException nfe)
    { // do nothing - fall through and return -1
    } // end catch

    return -1;

  } // end getCategoryParam

  /*--------------------------------------------------------------------------------
   * Overrides from class HttpServlet
   *--------------------------------------------------------------------------------
   */

  public String getServletInfo()
  {
    String rc = "Find servlet - The master search page\n"
              + "Part of the Venice Web Communities System\n";
    return rc;

  } // end getServletInfo

  /*--------------------------------------------------------------------------------
   * Overrides from class VeniceServlet
   *--------------------------------------------------------------------------------
   */

  protected VeniceContent doVeniceGet(HttpServletRequest request, VeniceEngine engine,
				      UserContext user, RenderData rdat)
    throws ServletException, IOException, VeniceServletResult
  {
    changeMenuTop(request);  // we go to the "top" menus for this

    // figure out which page to display
    int disp = getDisplayParam(request,FindData.getNumChoices());
    FindData finddata = new FindData(engine,user,disp);

    // figure out the category ID parameter
    int cat = -1;
    if (disp==FindData.FD_COMMUNITIES)
      cat = getCategoryParam(engine,request);

    try
    { // attempt to configure the display
      finddata.loadGet(cat);

    } // end try
    catch (DataException de)
    { // database error, man
      return new ErrorBox("Database Error","Database error accessing category data: " + de.getMessage(),"top");

    } // end catch

    setMyLocation(request,"find?" + request.getQueryString());
    return finddata;

  } // end doVeniceGet

  protected VeniceContent doVenicePost(HttpServletRequest request, VeniceEngine engine,
				       UserContext user, RenderData rdat)
    throws ServletException, IOException, VeniceServletResult
  {
    changeMenuTop(request);  // we go to the "top" menus for this

    // figure out which page to display
    int disp = getDisplayParam(request,FindData.getNumChoices());
    FindData finddata = new FindData(engine,user,disp);

    try
    { // attempt to configure the display
      finddata.loadPost(request);

    } // end try
    catch (ValidationException ve)
    { // error in find parameters
      return new ErrorBox("Find Error",ve.getMessage(),"top");

    } // end catch
    catch (DataException de)
    { // database error, man
      return new ErrorBox("Database Error","Database error on find: " + de.getMessage(),"top");

    } // end catch

    setMyLocation(request,"find?disp=" + finddata.getDisplayOption());
    return finddata;

  } // end doVenicePost

} // end class Find
